"""Widget that contains the widgets of the left column."""
from __future__ import annotations

import logging
from typing import List, Optional, Tuple, Type, TypeVar, cast

from PyQt5.QtCore import pyqtSignal
from PyQt5.QtWidgets import (
    QFrame,
    QHBoxLayout,
    QLabel,
    QLayout,
    QStackedWidget,
    QVBoxLayout,
    QWidget,
)

from qute_style.widgets.base_widgets import BaseWidget, SettingsBaseWidget
from qute_style.widgets.icon import Icon
from qute_style.widgets.left_column_close_button import LeftColumnCloseButton

log = logging.getLogger(
    f"qute_style.{__name__}"
)  # pylint: disable=invalid-name

BaseWidgetT = TypeVar("BaseWidgetT", bound=BaseWidget)


class LeftColumn(QWidget):
    """Widget that contains the widgets of the left column."""

    close_column = pyqtSignal(name="close_column")

    def __init__(
        self,
        app_parent: QWidget,
        widget_types: List[Type[BaseWidget]],
        parent: QWidget | None = None,
    ):
        """Create a new LeftColumn."""
        super().__init__(parent)
        # Create all widgets that shall be displayed in the left column.
        log.debug("Creating LeftColumn with widgets %s", widget_types)
        self._widgets = [widget_type() for widget_type in widget_types]

        # Create the main layout.
        base_layout = QVBoxLayout(self)

        # Set the top margin to 4 so that the header will be aligned with the
        # menu button's focus frame and the title bar.
        base_layout.setContentsMargins(0, 4, 0, 0)
        base_layout.setSpacing(0)

        # Create the title frame that contains icon, text and close button.
        self._icon, self._title_label = self._create_title_frame(
            app_parent, base_layout
        )

        # Create the content frame with a QStackedWidget containing all widgets
        # in widget_types.
        self._stacked_widget, self._widgets = self.create_content_frame(
            base_layout, widget_types
        )

        # Set the first widget so that title text is set.
        if widget_types:
            self.set_column_widget(widget_types[0])

    @staticmethod
    def create_content_frame(
        base_layout: QLayout,
        widget_types: List[Type[BaseWidget]],
    ) -> Tuple[QStackedWidget, List[BaseWidget]]:
        """
        Create the content QFrame and add it to the given layout.

        The frame contains a layout that wraps a QStackedWidget with a margin
        of 5 pixels on each side. The QStackedWidget contains all widgets that
        will be shown in the column when pressing the respective buttons in the
        LeftMenu.

        This method also creates the widgets in widget_types and returns them
        together with the QStackedWidget.
        """
        content_frame = QFrame()
        base_layout.addWidget(content_frame)
        main_pages_layout = QVBoxLayout(content_frame)
        main_pages_layout.setContentsMargins(5, 5, 5, 5)
        stacked_widget = QStackedWidget(content_frame)
        main_pages_layout.addWidget(stacked_widget)
        widgets = [widget_type() for widget_type in widget_types]
        for widget in widgets:
            stacked_widget.addWidget(widget)
        return stacked_widget, widgets

    def _create_title_frame(
        self, app_parent: QWidget, layout: QLayout
    ) -> Tuple[Icon, QLabel]:
        """
        Create the title frame containing icon, title and close button.

        Adds the title frame to the given layout.
        """
        # Create the QFrame that contains all elements.
        title_frame = QFrame()
        title_frame.setFixedHeight(47)
        title_base_layout = QVBoxLayout(title_frame)
        title_base_layout.setContentsMargins(5, 3, 5, 3)

        # Create a background QFrame that paints the box around the elements.
        title_bg_frame = QFrame()
        title_bg_frame.setObjectName("title_bg_frame")
        title_bg_layout = QHBoxLayout(title_bg_frame)
        title_bg_layout.setContentsMargins(5, 5, 5, 5)
        title_bg_layout.setSpacing(3)

        # Add the Icon that is painted in the given color.
        icon = Icon()
        title_bg_layout.addWidget(icon)

        # Add the title QLabel.
        title_label = QLabel()
        title_label.setObjectName("column_title_label")
        title_bg_layout.addWidget(title_label)

        # Add the button to close the column.
        btn_close = LeftColumnCloseButton(
            app_parent,
            tooltip_text=self.tr("Schließen"),
            icon_path=":/svg_icons/close.svg",
        )
        btn_close.clicked.connect(self.close_column)
        btn_close.setFixedSize(30, 30)
        title_bg_layout.addWidget(btn_close)

        # Add the title background QFrame to the title's layout.
        title_base_layout.addWidget(title_bg_frame)

        # Add the title QFrame to the layout.
        layout.addWidget(title_frame)

        # Return Icon and title QLabel
        return icon, title_label

    def handle_settings_display(
        self, settings_widget: None | QWidget, icon: str
    ) -> None:
        """Handle the display of the settings widget."""
        current_type = self.current_widget_type()
        assert current_type is not None

        # Check if settings are displayed
        # Do not change to isinstance
        if issubclass(current_type, SettingsBaseWidget):
            current_widget = self.widget(current_type)
            # Clear settings
            # clear_widget() and add_widget() are methods of SettingsBaseWidget
            current_widget.clear_widget()

            # Set general settings icon
            self._icon.set_icon(current_widget.ICON)

            # If specific settings, add widget and set specific Icon
            if settings_widget:
                current_widget.add_widget(settings_widget)
                self._icon.set_icon(icon)

    def set_column_widget(self, widget_type: Type[BaseWidget]) -> None:
        """Set left column pages."""
        log.debug("Setting current widget to: %s", widget_type)
        widget = self.widget(widget_type)
        self._stacked_widget.setCurrentWidget(widget)
        self._title_label.setText(widget.NAME)
        self._icon.set_icon(widget.ICON)

    def widget(self, widget_type: Type[BaseWidgetT]) -> BaseWidgetT:
        """Get the widget of the given type."""
        for widget in self._widgets:
            if isinstance(widget, widget_type):
                return widget
        raise ValueError(  # pragma: no cover
            f"Could not find widget {widget_type}"
        )

    def current_widget_type(self) -> Optional[Type[BaseWidget]]:
        """Return the currently active widget class."""
        widget = self._stacked_widget.currentWidget()
        if widget:
            return cast(Type[BaseWidget], type(widget))
        # Return None explicitly instead of <class NoneType>
        return None



"""Development functions."""
# mypy: ignore-errors
from __future__ import annotations

import json
import pickle
import re
import subprocess
import sys
import xml.etree.ElementTree as ET
from collections import defaultdict
from pathlib import Path
from typing import Any, Dict, List, Tuple

from PyQt5 import uic  # pylint: disable=wrong-import-order


def compile_ui_files(src_folders: List[Path]) -> None:
    """
    Compile the ui files.

    Compile the ui files in src_folders and copy the created
    files to gen folder.
    """
    for folder in src_folders:
        for file in folder.iterdir():
            print(f"Converting file {file}")
            with file.open("r", encoding="utf-8") as source:
                new_file = file.parent.parent / "gen" / f"ui_{file.stem}.py"
                with new_file.open(
                    "w",
                    encoding="utf-8",
                ) as target:
                    uic.compileUi(source, target)
        # run black after ui files are created
        subprocess.run(
            [
                Path(sys.executable).parent / "black",
                "-l79",
                folder.parent / "gen",
            ],
            check=False,
        )
    print("Conversion finished")


def get_sorted_version_directories(changelog_path: Path) -> List[Path]:
    """Get all version directories and return them sorted as list."""
    directories = [item for item in changelog_path.iterdir() if item.is_dir()]
    return sorted(
        directories,
        reverse=True,
        key=lambda x: tuple(map(int, x.name.split("."))),
    )


# key = widget class name
# values = changelogs for widget according to language (de/en)
WidgetLogInfo = Dict[str, List[Dict[str, str]]]


def list_dd() -> Dict[Any, List[Any]]:
    """
    Implement module level method to allow pickling of defaultdict.

    See: https://stackoverflow.com/questions/16439301/cant-pickle-defaultdict
    """
    return defaultdict(list)


def get_change_log_data(
    app_name: str,
    change_log_path: Path,
) -> Dict[str, WidgetLogInfo]:
    """Read in all the changelog data per version."""
    print("Get changelog data")
    change_log_data: Dict[str, WidgetLogInfo] = defaultdict(list_dd)
    sorted_directories = get_sorted_version_directories(change_log_path)
    for version in sorted_directories:
        for file in version.glob("*.json"):
            try:
                widget, texts = _parse_change_log(file)
            except NotImplementedError:
                continue
            change_log_data[version.name][widget or app_name].append(texts)
    return dict(change_log_data)


def _parse_change_log(file: Path) -> Tuple[str, Dict[str, str]]:
    """Parse the widget name and the change log texts from a JSON file."""
    with file.open(encoding="utf-8") as handle:
        entry = json.loads(handle.read())
    if isinstance(entry, List):
        # old changelog data was stored as list. Do not add to logs
        raise NotImplementedError(
            "Parsing of old changelogs isn't implemented"
        )
    text = _parse_str(entry, "text")
    widget = _parse_str(entry, "widget")
    text_en = _parse_str(entry, "text_en")

    return widget, {"de": text, "en": text_en}


def _parse_str(entry: Dict[Any, Any], key: str) -> str:
    """Parse the text for the given key from the entry (dict from JSON)."""
    try:
        text = entry[key]
        assert isinstance(text, str)
    except KeyError as error:
        # in case entry is not available an exception is thrown
        print(f"Entry in log not found: {key}")
        raise KeyError from error
    except AssertionError as error:
        print(f"Entry has invalid type: {key}")
        raise KeyError from error
    return text


def _create_resource_file(  # pylint: disable=too-many-locals
    resource_file_path: Path,
    change_log_path: Path,
    change_log_data: Dict[str, WidgetLogInfo],
) -> None:
    """Create the resource file from data."""
    print("Creating new resources.qrc")
    resource_file = resource_file_path / "resources.qrc"
    resource_py = resource_file_path / "resources_cl.py"
    resource_file.unlink(True)

    # Store the changelog data (serialize)
    print("Store the changelog data ")
    change_log_data_path = resource_file_path / "change_log_data.pickle"
    with change_log_data_path.open("wb") as handle:
        pickle.dump(change_log_data, handle, protocol=pickle.HIGHEST_PROTOCOL)
    rcc = ET.Element("RCC")
    qrc = ET.SubElement(rcc, "qresource")
    ET.SubElement(qrc, "file").text = "change_log_data.pickle"

    # store added images in the resource file. Images are extracted from text
    # and text_en
    added_images = set()
    for version_data in change_log_data.values():
        for log_data in version_data.values():
            for lang in log_data:
                images = re.findall('src=":([^"]+)"', lang["de"])
                added_images.update(list(images))
                images = re.findall('src=":([^"]+)"', lang["en"])
                added_images.update(list(images))

    for image in added_images:
        image_path = Path(change_log_path).joinpath(image)
        ET.SubElement(qrc, "file", {"alias": image}).text = str(
            image_path.relative_to(resource_file_path)
        )
    tree = ET.ElementTree(rcc)
    tree.write(resource_file)

    print("Generating resource_rc.py with new resources.qrc")
    assert subprocess.call(f"pyrcc5 -o {resource_py} {resource_file}") == 0

    print("Deleting resources.qrc")
    resource_file.unlink()
    print("Delete dict data")
    change_log_data_path.unlink()


def generate_changelog_resource_file(
    app_name: str,
    change_log_path: Path,
    resource_file_path: Path,
) -> None:
    """
    Generate changelog resource file.

    app_name: Name of the application
    change_log_path: changelog path
    resource_file_path: path where the resource file should be stored
    """
    print("Generate changelog resource file")
    change_log_data = get_change_log_data(app_name, change_log_path)
    _create_resource_file(resource_file_path, change_log_path, change_log_data)



"""QuteStyle Library - helper: useful functions for QuteStyle tools."""
import logging
import sys
from pathlib import Path
from typing import List, Union, cast
from xml.etree.ElementTree import Element, SubElement, tostring

from PyQt5.QtCore import QBuffer, QByteArray, QIODevice, QObject
from PyQt5.QtGui import QColor, QPaintEvent, QPixmap
from pyqtspinner.spinner import WaitingSpinner

from qute_style.style import get_color

log = logging.getLogger("qute_style")


def check_ide() -> bool:
    """Check if the application is run from the IDE."""
    return Path(sys.argv[0]).suffix == ".py"


def encode_pixmap(pixmap: QPixmap) -> str:
    """Encode a QPixmap to a Base64 str."""
    byte_array = QByteArray()
    buffer = QBuffer(byte_array)
    buffer.open(QIODevice.WriteOnly)
    pixmap.save(buffer, "png")
    return byte_array.toBase64().data().decode("utf-8")


def decode_pixmap(pixmap_string: str) -> QPixmap:
    """Decode a QPixmap from a Base64 str."""
    byte_array = QByteArray.fromBase64(
        cast(QByteArray, pixmap_string.encode("utf-8"))
    )
    qpixmap = QPixmap()
    qpixmap.loadFromData(byte_array)
    return qpixmap


class StyledWaitingSpinner(WaitingSpinner):
    """Styled Version of QWaitingSpinner."""

    def paintEvent(self, _: QPaintEvent) -> None:
        """Overwrite method to change color of spinner."""
        self._color = QColor(get_color("context_color"))
        super().paintEvent(_)


def create_waiting_spinner(
    parent: QObject,
    number_of_lines: int = 28,
    line_length: int = 20,
    inner_radius: int = 15,
) -> StyledWaitingSpinner:
    """Create a waiting spinner with default config."""
    spinner = StyledWaitingSpinner(parent)
    spinner.color = get_color("context_color")
    spinner.number_of_lines = number_of_lines
    spinner.line_length = line_length
    spinner.inner_radius = inner_radius
    spinner.line_width = 2
    return spinner


def create_tooltip(title: str, description: Union[str, List[str]]) -> str:
    """Create a tooltip as HTML str."""
    top = Element("div")
    first = SubElement(top, "p")
    headline = SubElement(first, "b")
    headline.text = title
    second = SubElement(top, "p")
    text = SubElement(second, "small")
    if isinstance(description, str):
        text.text = description
    else:
        text.text = "<br />".join(description)
    return tostring(top).decode("utf-8")



"""HomePage for qute_style Apps."""
from __future__ import annotations

import pickle
from enum import IntEnum
from typing import Callable, Dict, List, Tuple, Type, cast

from PyQt5.QtCore import (
    QEasingCurve,
    QFile,
    QIODevice,
    QPropertyAnimation,
    QSettings,
    QSize,
    Qt,
    pyqtSignal,
    pyqtSlot,
)
from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtSvg import QSvgWidget
from PyQt5.QtWidgets import (
    QGridLayout,
    QLabel,
    QLayout,
    QPushButton,
    QScrollArea,
    QSizePolicy,
    QSpacerItem,
    QStackedWidget,
    QVBoxLayout,
    QWidget,
)

from qute_style.style import THEMES, _create_theme_drawing, log
from qute_style.widgets.base_widgets import MainWidget
from qute_style.widgets.icon import Icon


class WidgetType(IntEnum):
    """Definition of WidgetType."""

    HOMEPAGE = 0
    VERSION_HISTORY = 1
    STYLE_WIDGET = 2


class StackedWidget(QStackedWidget):
    """Stacked widget for homepage."""

    widget_selected = pyqtSignal(int, name="widget_selected")

    def __init__(self, parent: QWidget | None = None) -> None:
        """Create a new StackedWidget."""
        super().__init__(parent)
        self._animation_running = False
        self._animation = QPropertyAnimation(self, b"size")
        self._animation.setDuration(400)
        self._animation.setEasingCurve(QEasingCurve.OutQuad)
        self._animation.finished.connect(self.on_animation_finished)

    def set_current_index(self, index: int, animate: bool = True) -> None:
        """Set page index."""
        if self._animation_running or index == self.currentIndex():
            # if index is changed during animation skip and revert to old index
            # if selection is current selection do not start animation
            self.widget_selected.emit(self.currentIndex())
            return
        super().setCurrentIndex(index)
        self.widget_selected.emit(index)
        if animate:
            self._animation.setStartValue(
                QSize(self.currentWidget().width(), 0)
            )
            self._animation.setEndValue(
                QSize(
                    self.currentWidget().width(), self.currentWidget().height()
                )
            )
            self._animation.start()
            self._animation_running = True

    @pyqtSlot(name="on_animation_finished")
    def on_animation_finished(self) -> None:
        """Animation finished."""
        self._animation_running = False


class HomePage(MainWidget):
    """HomePage for qute_style Apps."""

    ICON = ":/svg_icons/home.svg"
    NAME = "Information"

    change_theme = pyqtSignal(str, name="change_theme")

    def __init__(
        self,
        app_info: Tuple[str, str, str],
        visible_widgets: List[Type[MainWidget]],
        parent: QWidget | None = None,
    ) -> None:
        """Init Homepage."""
        super().__init__(parent)
        self._visible_widgets: List[Type[MainWidget]] = visible_widgets
        self._app_name, self._app_logo, self._app_lang = app_info
        self._widget_stack = StackedWidget()
        self._select_buttons: Dict[int, QPushButton] = {}
        self.setLayout(self._create_layout(self._create_welcome_widget()))
        if self._check_show_theme_selection_widget():
            self._widget_stack.set_current_index(
                WidgetType.STYLE_WIDGET, False
            )
        else:
            self._widget_stack.set_current_index(WidgetType.HOMEPAGE, False)

    def _create_welcome_widget(self) -> QWidget:
        """Add custom widget to as homepage."""
        widget = QWidget()
        layout = QGridLayout(widget)
        layout.setVerticalSpacing(100)
        widget.setLayout(layout)
        layout.addItem(
            QSpacerItem(0, 0, QSizePolicy.Minimum, QSizePolicy.Expanding)
        )

        label = QLabel(f"Willkommen zur {self._app_name}")
        label.setObjectName("heading_label")
        layout.addWidget(label, 1, 0)
        layout.setAlignment(label, Qt.AlignCenter)

        logo_svg = QSvgWidget(self._app_logo)
        logo_svg.setFixedSize(QSize(200, 200))
        layout.addWidget(logo_svg, 2, 0)
        layout.setAlignment(logo_svg, Qt.AlignCenter)

        layout.addItem(
            QSpacerItem(3, 0, QSizePolicy.Minimum, QSizePolicy.Expanding)
        )

        return widget

    @staticmethod
    def _check_show_theme_selection_widget() -> bool:
        """
        Check if theme selection widget should be displayed.

        Theme selection should be displayed only once at first start of app.
        """
        show_selection = cast(
            bool,
            QSettings().value("CustomThemeSelectionActive", True, type=bool),
        )
        QSettings().setValue("CustomThemeSelectionActive", False)
        return show_selection

    def _create_layout(self, welcome_widget: QWidget) -> QLayout:
        """Create layout."""
        self._widget_stack.addWidget(welcome_widget)
        self._widget_stack.addWidget(self._create_version_history_widget())
        self._widget_stack.addWidget(self._create_style_selection_widget())
        self._widget_stack.widget_selected.connect(self.on_index_changed)

        homepage_btn = QPushButton(self.tr("Homepage"))
        homepage_btn.setCheckable(True)
        homepage_btn.setAutoExclusive(True)
        self._select_buttons[WidgetType.HOMEPAGE] = homepage_btn
        version_history_btn = QPushButton(self.tr("Versionshistorie"))
        version_history_btn.setCheckable(True)
        version_history_btn.setAutoExclusive(True)
        self._select_buttons[WidgetType.VERSION_HISTORY] = version_history_btn
        stylesheet_btn = QPushButton(self.tr("Auswahl Style"))
        stylesheet_btn.setCheckable(True)
        stylesheet_btn.setAutoExclusive(True)
        self._select_buttons[WidgetType.STYLE_WIDGET] = stylesheet_btn

        homepage_btn.clicked.connect(
            lambda: self._widget_stack.set_current_index(WidgetType.HOMEPAGE)
        )
        version_history_btn.clicked.connect(
            lambda: self._widget_stack.set_current_index(
                WidgetType.VERSION_HISTORY
            )
        )
        stylesheet_btn.clicked.connect(
            lambda: self._widget_stack.set_current_index(
                WidgetType.STYLE_WIDGET
            )
        )
        layout = QGridLayout(self)
        layout.addWidget(self._widget_stack, 0, 0, 1, 3)
        layout.addWidget(homepage_btn, 1, 0)
        layout.addWidget(version_history_btn, 1, 1)
        layout.addWidget(stylesheet_btn, 1, 2)
        return layout

    @pyqtSlot(int, name="on_index_changed")
    def on_index_changed(self, index: int) -> None:
        """
        On index changed.

        Set the correct button if a stacked widget is selected.
        """
        self._select_buttons[index].setChecked(True)

    def _create_version_history_widget(self) -> QWidget:
        """Create the version history."""
        resource_path = ":/change_log_data.pickle"
        resource_file = QFile(resource_path)
        change_log_data: Dict[str, Dict[str, List[Dict[str, str]]]] = {}
        if resource_file.open(QIODevice.ReadOnly):
            pickle_data = resource_file.readAll()
            change_log_data = pickle.loads(pickle_data.data())
            resource_file.close()
        else:
            log.debug("No changelog data found! %s", resource_path)

        widget = QWidget()
        vert_box_layout = QVBoxLayout(widget)
        widget.setLayout(vert_box_layout)
        label = QLabel(self.tr("Versionshistorie"))
        label.setObjectName("heading_label")
        label.setContentsMargins(0, 0, 0, 10)
        vert_box_layout.addWidget(label)
        vert_box_layout.setAlignment(label, Qt.AlignCenter)

        scroll_area = QScrollArea()
        scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        vert_box_layout.addWidget(scroll_area)
        scroll_area.setWidgetResizable(True)
        central_widget = QWidget()
        central_widget.setObjectName("style_selection_widget")

        grid = QGridLayout()
        grid.setHorizontalSpacing(0)
        grid.setVerticalSpacing(20)
        central_widget.setLayout(grid)
        scroll_area.setWidget(central_widget)
        self.fill_version_info(grid, change_log_data, self._visible_widgets)
        return widget

    def fill_version_info(
        self,
        grid_layout: QGridLayout,
        change_log_data: Dict[str, Dict[str, List[Dict[str, str]]]],
        visible_widgets: List[Type[MainWidget]],
    ) -> None:
        """Set up version grid from change_log_data."""
        log.debug("Fill version grid")
        # get the sorted version keys in descending order. Newest version first
        if not change_log_data:
            log.debug("No changelog data found")
            label = QLabel(self.tr("Keine Einträge vorhanden"))
            label.setObjectName("heading1_label")
            label.setMinimumHeight(30)
            grid_layout.addWidget(label, 0, 1)
            grid_layout.setAlignment(label, Qt.AlignCenter)
            return

        grid_layout.addItem(
            QSpacerItem(0, 0, QSizePolicy.Expanding, QSizePolicy.Minimum),
            0,
            3,
        )
        grid_layout.addItem(
            QSpacerItem(0, 0, QSizePolicy.Expanding, QSizePolicy.Minimum),
            0,
            0,
        )

        row = 0
        for version, log_data_dict in change_log_data.items():
            label = QLabel(f"V {version}")
            label.setObjectName("heading1_label")
            label.setMinimumHeight(30)
            grid_layout.addWidget(label, row, 1, 1, 2)
            row += 1

            # handle version specific logs
            entries_available: bool = False
            if self._app_name in log_data_dict:
                # handle general app logs
                w_icon = self._app_logo
                w_name = self._app_name
                entries_available = True
                row = self._add_item_to_grid(
                    grid_layout,
                    row,
                    (w_name, w_icon),
                    [
                        dict[self._app_lang]
                        for dict in log_data_dict[self._app_name]
                    ],
                )

            # handle widget specific logs
            for widget in visible_widgets:
                if widget.__name__ not in log_data_dict:
                    # widget not present anymore. Ignore
                    continue
                w_icon = widget.ICON
                w_name = widget.NAME
                entries_available = True
                row = self._add_item_to_grid(
                    grid_layout,
                    row,
                    (w_name, w_icon),
                    [
                        dict[self._app_lang]
                        for dict in log_data_dict[widget.__name__]
                    ],
                )

            if not entries_available:
                label = QLabel(
                    self.tr(
                        "Keine relevanten Änderungen für Ihre "
                        "Benutzergruppe vorhanden."
                    )
                )
                label.setMinimumHeight(30)
                grid_layout.addWidget(label, row, 2)
                row += 1

        grid_layout.addItem(
            QSpacerItem(0, 0, QSizePolicy.Minimum, QSizePolicy.Expanding),
            row,
            0,
            -1,
            0,
        )

    def _add_item_to_grid(
        self,
        grid_layout: QGridLayout,
        row: int,
        widget_info: Tuple[str, str],
        text_items: List[str],
    ) -> int:
        """Add changelog item to grid."""
        widget_name, widget_icon = widget_info
        if widget_name == self._app_name:
            # load as svg icon
            svg_widget = QSvgWidget(widget_icon)
            svg_widget.setFixedSize(QSize(18, 18))
            grid_layout.addWidget(svg_widget, row, 1)
        else:
            icon = Icon(radius=18)
            icon.set_icon(widget_icon)
            grid_layout.addWidget(icon, row, 1)

        label = QLabel(widget_name)
        label.setObjectName("heading2_label")
        label.setMinimumHeight(30)
        grid_layout.addWidget(label, row, 2)
        row += 1
        for text in text_items:
            label = QLabel(text)
            label.setWordWrap(True)
            label.setMinimumHeight(30)
            label.setMinimumWidth(700)
            label.setMaximumWidth(1000)
            grid_layout.addWidget(label, row, 2)
            row += 1
        return row

    def _create_style_selection_widget(  # pylint: disable=too-many-locals
        self,
    ) -> QWidget:
        """Add style selection."""
        widget = QWidget()
        vert_box_layout = QVBoxLayout()
        vert_box_layout.setSpacing(20)
        widget.setLayout(vert_box_layout)

        label = QLabel(f"Willkommen zur {self._app_name}")
        label.setObjectName("heading_label")
        label.setContentsMargins(0, 0, 0, 20)
        vert_box_layout.addWidget(label)
        vert_box_layout.setAlignment(label, Qt.AlignCenter)

        label = QLabel(
            self.tr(
                "Bitte wählen Sie ein ToolBox-Design aus, "
                "das Sie verwenden wollen."
            ),
            self,
        )
        label.setObjectName("heading_label")
        vert_box_layout.addWidget(label)
        vert_box_layout.setAlignment(label, Qt.AlignCenter)
        label = QLabel(
            self.tr(
                "(Der Design-Stil kann jederzeit über die "
                "Einstellungen geändert werden.)"
            ),
            self,
        )
        vert_box_layout.addWidget(label)
        vert_box_layout.setAlignment(label, Qt.AlignCenter)

        scroll_area = QScrollArea(self)
        vert_box_layout.addWidget(scroll_area)
        scroll_area.setWidgetResizable(True)
        central_widget = QWidget()
        central_widget.setObjectName("style_selection_widget")

        grid = QGridLayout()
        grid.setHorizontalSpacing(40)
        grid.setVerticalSpacing(20)
        central_widget.setLayout(grid)
        scroll_area.setWidget(central_widget)

        def select_theme(theme_name: str) -> None:
            """Select the theme."""
            self.change_theme.emit(theme_name)

        def update_lambda(theme_name: str) -> Callable[[], None]:
            """Update lambda."""
            return lambda: select_theme(theme_name)

        icon_size = QSize(300, 200)
        grid.addItem(
            QSpacerItem(0, 0, QSizePolicy.Expanding, QSizePolicy.Minimum),
            0,
            0,
            -1,
            0,
        )
        grid.addItem(
            QSpacerItem(0, 0, QSizePolicy.Expanding, QSizePolicy.Minimum),
            0,
            3,
            -1,
            0,
        )
        grid_column = 1
        grid_row = 0
        for theme, color_names in THEMES.items():
            # check if the theme has a preview image otherwise don't use as
            # an option to select
            button = QPushButton(self)
            pixmap = QPixmap(_create_theme_drawing(icon_size, color_names))
            button.setIcon(QIcon(pixmap))
            button.setIconSize(icon_size)
            button.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
            if grid_column > 2:
                grid_row += 2
                grid_column = 1
            button.clicked.connect(update_lambda(theme))
            grid.addWidget(button, grid_row, grid_column)
            grid.addWidget(QLabel(theme, self), grid_row + 1, grid_column)
            grid_column += 1
        grid.addItem(
            QSpacerItem(0, 0, QSizePolicy.Minimum, QSizePolicy.Expanding),
            grid_row + 2,
            0,
            -1,
            0,
        )

        return widget



"""Credit bar with copyright text on the left and version on the right."""
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import (
    QFrame,
    QHBoxLayout,
    QLabel,
    QSizePolicy,
    QSpacerItem,
)


class CreditBar(QFrame):
    """Credit bar with copyright text on the left and version on the right."""

    def __init__(self, version: str) -> None:
        """Create a new CreditBar."""
        super().__init__()
        self.setObjectName("bg_two_frame")

        self.setFixedHeight(26)

        bg_layout = QHBoxLayout(self)
        bg_layout.setContentsMargins(0, 0, 0, 0)

        copyright_label = QLabel("By: Technical Support Local")
        copyright_label.setAlignment(Qt.AlignVCenter)

        version_label = QLabel(version)
        version_label.setAlignment(Qt.AlignVCenter)

        separator = QSpacerItem(
            20, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        bg_layout.addWidget(copyright_label)
        bg_layout.addSpacerItem(separator)
        bg_layout.addWidget(version_label)



"""Startup threads."""
from __future__ import annotations

import logging
from datetime import datetime
from typing import Tuple, Type

from PyQt5.QtCore import QThread

from qute_style.qs_main_window import AppData

log = logging.getLogger(
    f"qute_style.{__name__}"
)  # pylint: disable=invalid-name


class StartupThread(QThread):
    """StartupThread base class."""

    START_DEPENDS_ON: Tuple[Type[StartupThread], ...] = ()

    # Each startup thread must have a unique priority for its exit function.
    # The exit function of the thread with the lowest priority is executed
    # first in case there are more than one finished threads at the same time
    EXIT_FUNCTION_PRIORITY: int = 0

    def __init__(self, app_data: AppData):
        """Startupthread init function."""
        super().__init__(None)
        self.app_data = app_data
        log.info(
            "Creating %s for app %s version %s",
            self.__class__,
            self.app_data.app_name,
            self.app_data.app_version,
        )

    def run(self) -> None:
        """Thread run."""
        start_time = datetime.now()
        self._function_to_execute()
        log.debug(
            "Time taken %s: %s", self.__class__, datetime.now() - start_time
        )

    def _function_to_execute(self) -> None:
        """Thread function. Must be implemented by subclasses."""
        raise NotImplementedError  # pragma: no cover

    @property
    def exit_application(self) -> bool:
        """Return if the app should call exit_function on thread finished."""
        return False

    def exit_function(self) -> None:
        """Exit function to be called."""



"""ColorManager for live editing of colors."""
from __future__ import annotations

import json
import logging
from typing import Dict, cast

from PyQt5.QtCore import Qt, pyqtSignal
from PyQt5.QtGui import QColor, QPaintEvent
from PyQt5.QtWidgets import (
    QApplication,
    QColorDialog,
    QGridLayout,
    QHBoxLayout,
    QLabel,
    QScrollArea,
    QWidget,
)

from qute_style.style import MAIN_STYLE, THEMES, get_current_style
from qute_style.widgets.base_widgets import BaseWidget
from qute_style.widgets.icon_button import IconButton

log = logging.getLogger(
    f"qute_style.{__name__}"
)  # pylint: disable=invalid-name


class SmallIconButton(IconButton):
    """Icon button with a reduced size."""

    FIXED_WIDTH = 24
    FIXED_HEIGHT = 24


class ColorWidget(QWidget):
    """Widget displaying a color with a button to change it."""

    color_changed = pyqtSignal(name="color_changed")

    def __init__(
        self, key: str, color: str, parent: QWidget | None = None
    ) -> None:
        """Create a new Color Widget."""
        super().__init__(parent)
        self.key = key
        layout = QHBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        self._color_label = QLabel(color)
        self._color_label.setTextInteractionFlags(Qt.TextSelectableByMouse)
        layout.addWidget(self._color_label)
        self._color_button = SmallIconButton(self, ":/svg_icons/palette.svg")
        self._color_button.clicked.connect(self.on_change_color)
        layout.addWidget(self._color_button)

    @property
    def color(self) -> str:
        """Return the current color."""
        return self._color_label.text()

    @color.setter
    def color(self, color: str) -> None:
        """Set the color."""
        log.debug(
            "Setting new color: %s, old color: %s",
            color,
            self._color_label.text(),
        )
        self._color_label.setText(color.lower())
        self._color_label.setStyleSheet(
            f"QLabel {{ background-color: {color} ; color: black}}"
        )

    def on_change_color(self) -> None:
        """Open a color dialog to change the color."""
        dialog = QColorDialog(QColor(self._color_label.text()), self)
        if dialog.exec() == QColorDialog.Accepted:
            self.color = dialog.selectedColor().name()
            self.color_changed.emit()


class ColorManager(BaseWidget):
    """ColorManager for live editing of colors."""

    ICON = ":/svg_icons/palette.svg"
    NAME = "Color Manager"

    def __init__(self, parent: QWidget | None = None) -> None:
        """Create a new ColorManager."""
        super().__init__(parent)
        layout = QHBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        scroll_area = QScrollArea(self)
        scroll_area.setObjectName("bg_two_frame")
        scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        layout.addWidget(scroll_area)

        frame = QWidget()
        frame.setObjectName("bg_two_frame")
        grid_layout = QGridLayout(frame)
        assert frame.layout() is grid_layout
        scroll_area.setWidget(frame)
        grid_layout.setContentsMargins(0, 0, 0, 0)
        grid_layout.setSizeConstraint(QGridLayout.SetFixedSize)
        grid_layout.setSpacing(2)

        self._widgets = []

        for idx, key in enumerate(sorted(THEMES[list(THEMES.keys())[0]])):
            log.debug("Adding row for key %s", key)
            grid_layout.addWidget(QLabel(key), idx, 0)
            widget = ColorWidget(key, "", scroll_area)
            widget.color_changed.connect(self.on_color_changed)
            grid_layout.addWidget(widget, idx, 1)
            self._widgets.append(widget)

        self._code_label = QLabel(self)
        self._code_label.setTextInteractionFlags(Qt.TextSelectableByMouse)
        grid_layout.addWidget(
            self._code_label, grid_layout.rowCount(), 0, 1, 2
        )

        self._current_style: str = get_current_style()
        self.update_style()
        self.set_code()

    def paintEvent(  # pylint: disable=invalid-name
        self, event: QPaintEvent
    ) -> None:
        """Check if the style has changed."""
        if self._current_style != get_current_style():
            log.debug("Updating colors to style: %s", self._current_style)
            self._current_style = get_current_style()
            self.update_style()
        super().paintEvent(event)

    def update_style(self) -> None:
        """Update all widgets with the current set style."""
        style = THEMES[self._current_style]
        for widget in self._widgets:
            widget.color = style[widget.key]

    def _create_theme(self) -> Dict[str, str]:
        """Create the code from the selected colors."""
        return {widget.key: widget.color for widget in self._widgets}

    def on_color_changed(self) -> None:
        """Set the new stylesheet when a color has changed."""
        log.debug("Color has changed, changing theme.")
        theme = self._create_theme()
        app = cast(QApplication, QApplication.instance())
        assert app is not None
        app.activeWindow().setStyleSheet(MAIN_STYLE.format(**theme))
        self.set_code()

    def set_code(self) -> None:
        """Set the theme's code in the label."""
        theme = self._create_theme()
        self._code_label.setText(json.dumps(theme, sort_keys=True, indent=4))



"""QuteStyleApplication."""
from __future__ import annotations

import logging
import operator
from copy import copy
from typing import List, Type, cast

from PyQt5 import QtCore
from PyQt5.QtCore import QRectF, QSize, Qt, pyqtSlot
from PyQt5.QtGui import (
    QCloseEvent,
    QColor,
    QMouseEvent,
    QPainter,
    QPainterPath,
    QPixmap,
)
from PyQt5.QtSvg import QSvgWidget
from PyQt5.QtWidgets import (
    QApplication,
    QGridLayout,
    QLabel,
    QMainWindow,
    QSplashScreen,
)

from qute_style.helper import check_ide, create_waiting_spinner
from qute_style.qs_main_window import AppData, CustomMainWindow
from qute_style.startup_threads import StartupThread
from qute_style.style import get_color, get_style

log = logging.getLogger(
    f"qute_style.{__name__}"
)  # pylint: disable=invalid-name


class CustomSplashScreen(QSplashScreen):
    """Custom Splash screen for startup of app."""

    def __init__(self, app_data: AppData) -> None:
        """Init CustomSplashScreen."""
        super().__init__()
        style_sheet = get_style()
        self.setStyleSheet(style_sheet)
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
        background_color = get_color("bg_one")
        # use a pixmap as background
        size = QSize(350, 200)
        self.setFixedSize(size)
        pixmap = QPixmap(size)
        pixmap.fill(Qt.transparent)
        painter = QPainter(pixmap)
        painter.setRenderHint(QPainter.Antialiasing)
        path = QPainterPath()
        path.addRoundedRect(QRectF(0, 0, size.width(), size.height()), 12, 12)
        painter.fillPath(path, QColor(background_color))
        painter.end()
        # working solution to get the rounded corners to a splash screen
        self.setPixmap(pixmap)
        grid_layout = QGridLayout(self)
        grid_layout.setAlignment(Qt.AlignHCenter)
        logo_svg = QSvgWidget(app_data.app_splash_icon, self)
        logo_svg.setFixedSize(QSize(140, 140))
        grid_layout.addWidget(logo_svg, 0, 0, 1, 0, alignment=Qt.AlignCenter)
        grid_layout.addWidget(
            QLabel(f"Starting {app_data.app_name}. Please wait", self), 1, 0
        )
        label = QLabel(self)
        label.setFixedSize(30, 30)
        self._spinner = create_waiting_spinner(label, 18, 10, 5)
        self._spinner.start()
        grid_layout.addWidget(label, 1, 1)

    def closeEvent(  # pylint: disable=invalid-name
        self, close_event: QCloseEvent
    ) -> None:
        """Handle close event."""
        self._spinner.stop()
        super().closeEvent(close_event)

    def mousePressEvent(  # pylint: disable=invalid-name
        self, event: QMouseEvent
    ) -> None:
        """Disable default "click-to-dismiss" behaviour."""


class QuteStyleApplication(  # pylint: disable=too-many-instance-attributes
    QApplication
):
    """QuteStyleApplication."""

    MAIN_WINDOW_CLASS: Type[CustomMainWindow]

    STARTUP_THREADS: List[Type[StartupThread]] = []

    APP_DATA: AppData

    def __init__(self, argv: List[str], show_splash: bool = True) -> None:
        """Init QuteStyleApplication."""
        super().__init__(argv)

        self._update = "-u" not in argv and not check_ide()
        self._force_whats_new = "-w" in argv
        self._reset_settings = "-c" in argv

        self.check_startup_thread_configuration()
        self.setApplicationName(self.APP_DATA.app_name)
        self.setOrganizationName(self.APP_DATA.organization_name)
        self.setOrganizationDomain(self.APP_DATA.organization_domain)
        if show_splash:
            self._splash_screen: QSplashScreen | None = CustomSplashScreen(
                self.APP_DATA
            )
            self._splash_screen.show()
        else:
            self._splash_screen = None
        self._main_window: QMainWindow | None = None

        self._threads_to_run: List[Type[StartupThread]] = copy(
            self.STARTUP_THREADS
        )

        # need to hold a reference to the threads until finished
        self._threads_running: List[StartupThread] = []
        self._threads_finished: List[StartupThread] = []

        self._handle_startup_threads()

    def check_startup_thread_configuration(self) -> None:
        """
        Check the configuration of the startup threads.

        In case exit application function is overriden check for a unique
        priority of EXIT_FUNCTION_PRIORITY.
        """
        thread_priorities: List[int] = []
        for thread in self.STARTUP_THREADS:
            if id(StartupThread.exit_application) != id(
                thread.exit_application
            ):
                assert (
                    thread.EXIT_FUNCTION_PRIORITY > 0
                ), "EXIT_FUNCTION_PRIORITY of startup threads must be > 0"
                assert (
                    thread.EXIT_FUNCTION_PRIORITY not in thread_priorities
                ), "EXIT_FUNCTION_PRIORITY of startup threads must be unique"
                thread_priorities.append(thread.EXIT_FUNCTION_PRIORITY)

    def _handle_startup_threads(self) -> None:
        """Handle startup threads if any."""
        # reimplement this method to handle starting threads or for
        # example removing threads before start
        self.show_main_window()

    def _run_threads(self) -> None:
        """Start processing the startup threads."""
        for thread_class in copy(self._threads_to_run):
            log.debug("Preparing thread to start: %s", thread_class)

            threads_finished = [
                thread.__class__ for thread in self._threads_finished
            ]
            if any(
                dependency not in threads_finished
                for dependency in thread_class.START_DEPENDS_ON
            ):
                log.debug("Cannot run thread, dependencies not met")
                continue
            thread = thread_class(self.APP_DATA)
            self._threads_running.append(thread)
            self._threads_to_run.remove(thread_class)
            thread.finished.connect(self.on_finished_thread)
            thread.start()
            log.debug("Thread starting: %s", thread)

    @pyqtSlot(name="on_finished_thread")
    def on_finished_thread(self) -> None:
        """Handle a finished StartupThread."""
        thread = cast(StartupThread, self.sender())
        log.debug("Thread finished: %s", thread)
        self._threads_running.remove(thread)
        self._threads_finished.append(thread)
        if thread.exit_application:
            # clear all threads which are not started yet in case a thread
            # with exit is ended
            self._threads_to_run.clear()
        if not self._threads_running and not self._threads_to_run:
            # all running threads have finished. Check if there are threads
            # that exits the app
            exit_threads = [
                thread
                for thread in self._threads_finished
                if thread.exit_application
            ]
            # sort by EXIT_FUNCTION_PRIORITY
            exit_threads.sort(
                key=operator.attrgetter("EXIT_FUNCTION_PRIORITY")
            )
            self._threads_finished.clear()
            if exit_threads:
                log.debug("Running threads post process, Exit application.")
                exit_threads[0].exit_function()
                self.quit()
            else:
                log.debug(
                    "There are no threads left to run, "
                    "starting QuteStyleMainWindow."
                )
                self.show_main_window()
        else:
            log.debug("Threads still to run: %s", self._threads_to_run)
            log.debug("Thread currently running: %s", self._threads_running)
            self._run_threads()

    def show_main_window(self) -> None:
        """Show the main window."""
        self._main_window = self.MAIN_WINDOW_CLASS(
            self.APP_DATA, self._force_whats_new, self._reset_settings
        )
        if self._splash_screen:
            self._splash_screen.finish(self._main_window)
        self._main_window.show()



"""Custom widgets for grips to resize the application."""
import logging
from typing import TYPE_CHECKING

from PyQt5.QtCore import QRect, Qt, pyqtSignal
from PyQt5.QtGui import QCursor, QMouseEvent
from PyQt5.QtWidgets import QSizeGrip, QWidget

log = logging.getLogger(
    f"qute_style.{__name__}"
)  # pylint: disable=invalid-name


class CornerGrip(QSizeGrip):
    """Widget that shows a grip handle in a corner."""

    window_geometry_changed = pyqtSignal(QRect, name="window_geometry_changed")

    def __init__(self, parent: QWidget, position: Qt.Corner) -> None:
        """Create a new CornerGrip."""
        super().__init__(parent)
        self._position = position
        self.setObjectName("grip")
        self.setFixedSize(15, 15)

    if TYPE_CHECKING:

        def parent(self) -> QWidget:
            """Override base class method for correct type hint."""

    def adapt(self) -> None:
        """Adapt the position when the main window was resized."""
        if self._position == Qt.TopLeftCorner:
            self.move(5, 5)
        elif self._position == Qt.TopRightCorner:
            self.move(self.parent().width() - 20, 5)
        elif self._position == Qt.BottomLeftCorner:
            self.move(5, self.parent().height() - 20)
        elif self._position == Qt.BottomRightCorner:
            self.move(self.parent().width() - 20, self.parent().height() - 20)

    def mousePressEvent(  # pylint: disable=invalid-name
        self, _: QMouseEvent
    ) -> None:
        """
        Override the mousePressEvent.

        This is required to be overriden in this class
        otherwise the mouse move event is
        not working.
        """
        log.debug("Mouse press event")

    def mouseMoveEvent(  # pylint: disable=invalid-name
        self, event: QMouseEvent
    ) -> None:
        """
        Handle a mouse move event to resize the grip.

        The mouseMoveEvent method is only called after a mousePressEvent call.
        """
        self._resize_x_y(event.pos().x(), event.pos().y())
        super().mouseMoveEvent(event)

    def _resize_x_y(self, delta_x: int, delta_y: int) -> None:
        """
        Resize the parents geometry and by this reposition the grips.

        By doing the resize here flickering is avoided which comes with setting
        of Qt.WA_TranslucentBackground.
        """
        geo = self.parent().geometry()
        if self._position == Qt.TopLeftCorner:
            width = max(
                self.parent().minimumWidth(), self.parent().width() - delta_x
            )
            height = max(
                self.parent().minimumHeight(),
                self.parent().height() - delta_y,
            )
            geo.setLeft(geo.right() - width)
            geo.setTop(geo.bottom() - height)
        elif self._position == Qt.BottomLeftCorner:
            width = max(
                self.parent().minimumWidth(), self.parent().width() - delta_x
            )
            height = max(
                self.parent().minimumHeight(),
                self.parent().height() + delta_y,
            )
            geo.setLeft(geo.right() - width)
            geo.setHeight(height)
        elif self._position == Qt.TopRightCorner:
            width = max(
                self.parent().minimumWidth(), self.parent().width() + delta_x
            )
            height = max(
                self.parent().minimumHeight(),
                self.parent().height() - delta_y,
            )
            geo.setTop(geo.bottom() - height)
            geo.setWidth(width)
        elif self._position == Qt.BottomRightCorner:
            width = max(
                self.parent().minimumWidth(), self.parent().width() + delta_x
            )
            height = max(
                self.parent().minimumHeight(),
                self.parent().height() + delta_y,
            )
            geo.setHeight(height)
            geo.setWidth(width)
        self.window_geometry_changed.emit(geo)


class EdgeGrip(QWidget):
    """Widget that shows a grip handle in an edge."""

    window_geometry_changed = pyqtSignal(QRect, name="window_geometry_changed")

    def __init__(self, parent: QWidget, position: Qt.Edge) -> None:
        """Create a new EdgeGrip."""
        super().__init__(parent)
        self.installEventFilter(self.parent())
        self._position = position
        self.setObjectName("grip")
        if position in (Qt.TopEdge, Qt.BottomEdge):
            self.setCursor(QCursor(Qt.SizeVerCursor))
            self.setMaximumHeight(10)
        elif position in (Qt.LeftEdge, Qt.RightEdge):
            self.setCursor(QCursor(Qt.SizeHorCursor))
            self.setMaximumWidth(10)

    if TYPE_CHECKING:

        def parent(self) -> QWidget:
            """Override base class method for correct type hint."""

    def adapt(self) -> None:
        """Adapt the size and position when the main window was resized."""
        width = self.parent().width()
        height = self.parent().height()
        if self._position == Qt.TopEdge:
            self.setGeometry(5, 5, width, 10)
        elif self._position == Qt.BottomEdge:
            self.setGeometry(5, height - 15, width, 10)
        elif self._position == Qt.LeftEdge:
            self.setGeometry(5, 10, 10, height)
        elif self._position == Qt.RightEdge:
            self.setGeometry(width - 15, 10, 10, height)

    def mouseMoveEvent(  # pylint: disable=invalid-name
        self, event: QMouseEvent
    ) -> None:
        """
        Handle a mouse move event to resize the grip.

        The mouseMoveEvent method is only called after a mousePressEvent call.
        """
        if self._position in (Qt.TopEdge, Qt.BottomEdge):
            self._resize_y(event.pos().y())
        if self._position in (Qt.LeftEdge, Qt.RightEdge):
            self._resize_x(event.pos().x())
        event.accept()

    def _resize_x(self, delta_x: int) -> None:
        """Resize the grip in x-direction."""
        geo = self.parent().geometry()
        if self._position == Qt.LeftEdge:
            width = max(self.parent().minimumWidth(), geo.width() - delta_x)
            geo.setLeft(geo.right() - width)
        elif self._position == Qt.RightEdge:
            width = max(
                self.parent().minimumWidth(), self.parent().width() + delta_x
            )
            geo.setWidth(width)
        self.window_geometry_changed.emit(geo)

    def _resize_y(self, delta_y: int) -> None:
        """Resize the grip in y-direction."""
        geo = self.parent().geometry()
        if self._position == Qt.TopEdge:
            height = max(
                self.parent().minimumHeight(),
                geo.height() - delta_y,
            )
            geo.setTop(geo.bottom() - height)
        elif self._position == Qt.BottomEdge:
            height = max(
                self.parent().minimumHeight(),
                self.parent().height() + delta_y,
            )
            geo.setHeight(height)
        self.window_geometry_changed.emit(geo)



"""Icon that can be painted in any given color."""
from typing import Union, cast

from PyQt5 import QtGui
from PyQt5.QtGui import QPainter, QPixmap
from PyQt5.QtWidgets import QApplication, QWidget

from qute_style.style import get_color
from qute_style.widgets.custom_icon_engine import PixmapStore


class Icon(QWidget):
    """Icon that can be painted in any given color."""

    DEFAULT_ICON_PATH = ":/svg_icons/no_icon.svg"

    def __init__(
        self, radius: int = 20, color_name: Union[str, None] = "foreground"
    ) -> None:
        """Create a new Icon."""
        super().__init__()

        # Radius needs to be set
        self._radius = radius
        self.setFixedSize(int(1.5 * radius), int(1.5 * radius))
        self._icon_path = self.DEFAULT_ICON_PATH
        self._color_name = color_name

    def set_icon(self, icon_path: str) -> None:
        """Set the given icon path."""
        self._icon_path = icon_path
        self.update()

    @property
    def scale(self) -> float:
        """Return the current scale for painting."""
        # At this point, there must be a QApplication
        return cast(QApplication, QApplication.instance()).devicePixelRatio()

    def paintEvent(  # pylint: disable=invalid-name
        self, _: QtGui.QPaintEvent
    ) -> None:
        """Override QWidget.paintEvent to draw pixmap."""
        pixmap = self._get_pixmap()
        xy_pos = (self.height() - pixmap.height()) // 2

        painter = QPainter(self)
        painter.drawPixmap(xy_pos, xy_pos, pixmap)
        painter.end()

    def _get_pixmap(self) -> QPixmap:
        """Return the pixmap for painting."""
        color = get_color(self._color_name) if self._color_name else None
        radius = int(self._radius * self.scale)

        # Get the pixmap from store.
        pixmap = PixmapStore.inst().get_pixmap(
            self._icon_path, radius, radius, color
        )

        # Set the scale_factor -> displayed pixmap fits into target
        pixmap.setDevicePixelRatio(self.scale)
        return pixmap



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['qute_style',
 'qute_style.dev',
 'qute_style.gen',
 'qute_style.widgets',
 'qute_style_examples']

package_data = \
{'': ['*'],
 'qute_style': ['resources/png_icons/*',
                'resources/svg_icons/*',
                'resources/svg_images/*',
                'ui/*'],
 'qute_style_examples': ['example_images/*',
                         'test_changelog/1.01/*',
                         'test_changelog/1.10/*',
                         'test_changelog/1.100/*']}

install_requires = \
['PyQt5==5.15.6', 'pyqtspinner>=2.0.0,<3.0.0']

entry_points = \
{'console_scripts': ['qute-style-example = '
                     'qute_style_examples.main:main_method']}

setup_kwargs = {
    'name': 'qute-style',
    'version': '0.1.6',
    'description': 'QuteStyle is an expandable application framework for PyQt5',
    'long_description': '<p align="center">\n  <a href="https://github.com/TUV-SUD-Product-Service-GmbH/QuteStyle">\n    <img src="https://github.com/TUV-SUD-Product-Service-GmbH/QuteStyle/raw/master/qute_style/resources/svg_images/banner_qute_style.svg" alt="QuteStyle logo" width="500" height="200">\n  </a>\n</p>\n\n# QuteStyle\n\nQuteStyle is an expandable application framework for PyQt5 and heavily inspired by [PyDracula](https://github.com/Wanderson-Magalhaes/Modern_GUI_PyDracula_PySide6_or_PyQt6).\nThe main goal of this project is to provide a simple and easy to use application frame that can be used to create a new application.\nIt is mainly suited for applications that rely on a center widget for user interaction. Functionality is extendable by having different widgets that can be loaded into that center widget area.\n\n**Project status**\n\n[![Python Versions](https://img.shields.io/badge/Python-3.8%20|%203.9%20|%203.10-blue.svg?&logo=Python&logoWidth=18&logoColor=white)](https://www.python.org/downloads/)\n[![Qt Versions](https://img.shields.io/badge/Qt-5-blue.svg?&logo=Qt&logoWidth=18&logoColor=white)](https://www.qt.io/qt-for-python)\n[![License](https://img.shields.io/github/license/TUV-SUD-Product-Service-GmbH/QuteStyle.svg)](https://github.com/TUV-SUD-Product-Service-GmbH/QuteStyle/blob/master/LICENSE)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-black.svg)](https://github.com/python/black)\n\n**Tests**\n\n[![CodeQL](https://github.com/TUV-SUD-Product-Service-GmbH/QuteStyle/workflows/CodeQL/badge.svg)](https://github.com/TUV-SUD-Product-Service-GmbH/QuteStyle/actions?query=workflow%3ACodeQL)\n[![Build Status](https://github.com/TUV-SUD-Product-Service-GmbH/QuteStyle/workflows/Tests/badge.svg?branch=master&event=push)](https://github.com/TUV-SUD-Product-Service-GmbH/QuteStyle/actions?query=workflow%3ATests)\n[![Code Coverage](https://codecov.io/github/TUV-SUD-Product-Service-GmbH/QuteStyle/coverage.svg?branch=master&token=)](https://codecov.io/gh/TUV-SUD-Product-Service-GmbH/QuteStyle)\n[![pre-commit.ci status](https://results.pre-commit.ci/badge/github/TUV-SUD-Product-Service-GmbH/QuteStyle/master.svg)](https://results.pre-commit.ci/latest/github/TUV-SUD-Product-Service-GmbH/QuteStyle/master)\n\n## Features\n\n- Easy integration of already existing widgets\n- Preset themes that easily can be modified\n- Custom widgets\n- Splash screen\n- Build-in release history\n- Used and developed in a productive environment\n\n## Themes and Styled Widgets\n\nQuteStyle provides five themes, defining the color composition of the app.\nAdditionally, the user can define new themes ([check this out](https://github.com/TUV-SUD-Product-Service-GmbH/QuteStyle/blob/master/docs/style.md)). We provide five themes, for example a dark and light mode ```Darcula``` and ```Highbridge Grey```.\nWe defined [custom widgets](https://github.com/TUV-SUD-Product-Service-GmbH/QuteStyle/blob/master/docs/widgets.md), such that they fit to the overall style and implemented new behaviour. A selection can be found in the Test-App:\n\n<img src="https://raw.githubusercontent.com/TUV-SUD-Product-Service-GmbH/QuteStyle/master/qute_style_examples/example_images/highbridge_grey.PNG" alt="Highbridge Grey" width="400" height="300"><img src="https://raw.githubusercontent.com/TUV-SUD-Product-Service-GmbH/QuteStyle/master/qute_style_examples/example_images/darcula.PNG" alt="Darcula" width="400" height="300">\n\n\n## Requirements\n\n- [Python 3.8+](https://www.python.org/downloads/)\n- [PyQt5](https://pypi.org/project/PyQt5/)\n\n## Installation Method\n\n- Latest development version\n\n   ```plaintext\n   pip install qute-style\n   ```\n\n## Usage\n\n```Python\nimport sys\n\nfrom PyQt5.QtCore import Qt\nfrom PyQt5.QtWidgets import QApplication\n\nfrom qute_style_examples.sample_main_window import StyledMainWindow\nfrom qute_style.qs_application import QuteStyleApplication\nfrom qute_style.update_window import AppData\n\nclass MyApplication(QuteStyleApplication):\n    # take a look at qute_style_examples.sample_main_window and qute_style_examples.sample_widgets\n    # to find out more about setting up a main window and the widgets that it\n    # should display\n    MAIN_WINDOW_CLASS = StyledMainWindow\n    # add basic information about your application\n    APP_DATA = AppData(\n        "Test-App",\n        "2.3.4",\n        ":/svg_images/logo_qute_style.svg",\n        ":/svg_images/logo_qute_style.svg",\n        "",\n        "Test Version",\n    )\n\nif __name__ == "__main__":\n\n    APP_NAME = "Test-App"\n\n    # activate highdpi icons and scaling\n    QApplication.setAttribute(Qt.AA_EnableHighDpiScaling)\n    QApplication.setAttribute(Qt.AA_UseHighDpiPixmaps)\n\n    APP = MyApplication(sys.argv)\n    sys.exit(APP.exec())\n```\n\nFor further information, see our [documentation](https://github.com/TUV-SUD-Product-Service-GmbH/QuteStyle/tree/master/docs).\n\n## Example\n\nCheck out our example app by running:\n\n```plaintext\npython -m qute_style_examples\n```\n\n## License\n\nThe original design idea is from [Wanderson-Magalhaes](https://github.com/Wanderson-Magalhaes) and his project [PyDracula](https://github.com/Wanderson-Magalhaes/Modern_GUI_PyDracula_PySide6_or_PyQt6) (MIT License).\nThe svg files are derived from [Material design icons](https://fonts.google.com/icons) (Apache License Version 2.0). Other files are covered by QuteStyle\'s MIT license.\n\n## Contributing\n\nAll contributions, bug reports, bug fixes, documentation improvements, enhancements, and ideas are welcome.\n',
    'author': 'Marina Baumgartner, Dairen Gonschior, Tilman Krummeck, Dennis Spitzhorn, Gerhard Trapp, Patrick Zwerschke',
    'author_email': 'PS-TF-Entwicklung@tuev-sued.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/TUV-SUD-Product-Service-GmbH/QuteStyle',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""Styled main window sample."""
from __future__ import annotations

import logging

from PyQt5.QtCore import QSize, pyqtSignal, pyqtSlot
from PyQt5.QtWidgets import (
    QHBoxLayout,
    QLabel,
    QSizePolicy,
    QSpacerItem,
    QVBoxLayout,
    QWidget,
)

from qute_style.qs_main_window import AppData, QuteStyleMainWindow
from qute_style.style import THEMES, get_current_style
from qute_style.widgets.base_widgets import BaseWidget, SettingsBaseWidget
from qute_style.widgets.color_manager import ColorManager
from qute_style.widgets.home_page import HomePage
from qute_style.widgets.icon_button import IconButton
from qute_style_examples.sample_widgets import (
    ModelViewWidget,
    SpinnerWidget,
    TestWidget,
)

log = logging.getLogger(
    f"qute_style.{__name__}"
)  # pylint: disable=invalid-name


class SettingsWidget(SettingsBaseWidget):
    """Test SettingsWidget."""

    def __init__(self, parent: QWidget | None = None) -> None:
        """Create a new SettingsWidget."""
        super().__init__(parent)
        layout = QVBoxLayout(self)
        self.setLayout(layout)
        self.layout().addWidget(QLabel("Global settings"))
        self.layout().addItem(
            QSpacerItem(0, 0, QSizePolicy.Expanding, QSizePolicy.Expanding)
        )
        self._set_global_widget(self)


class InfoWidget(BaseWidget):
    """Test ColumnBaseWidget."""

    ICON = ":/svg_icons/info.svg"
    NAME = "Info"

    switch_style = pyqtSignal(name="switch_style")

    def __init__(self, parent: QWidget | None = None) -> None:
        """Create a new SettingsWidget."""
        super().__init__(parent)
        layout = QHBoxLayout(self)
        layout.addWidget(QLabel("Switch style"))
        button = IconButton(self, ":/svg_icons/send.svg")
        layout.addWidget(button)
        button.clicked.connect(self.switch_style)


class StyledMainWindow(QuteStyleMainWindow):
    """Test StyledMainWindow for validation of QuteStyle Darcula Style."""

    MAIN_WIDGET_CLASSES = [
        HomePage,
        TestWidget,
        ModelViewWidget,
        SpinnerWidget,
    ]
    RIGHT_WIDGET_CLASSES = [ColorManager]
    LEFT_WIDGET_CLASSES = [SettingsWidget, InfoWidget]

    MIN_SIZE = QSize(800, 600)

    def __init__(
        self,
        app_data: AppData,
        registry_reset: bool = False,
        force_whats_new: bool = False,
        parent: QWidget | None = None,
    ) -> None:
        """Create a new StyledMainWindow."""
        super().__init__(
            app_data,
            force_whats_new,
            registry_reset,
            parent,
        )
        self._left_column.widget(InfoWidget).switch_style.connect(
            self.on_switch_style
        )

        try:
            self._current_idx = (
                "Snow White",
                "Princess Pink",
                "Darcula",
                "Highbridge Gray",
                "Ruby Red",
            ).index(get_current_style())
        except KeyError:
            self._current_idx = 0

    @pyqtSlot(name="on_switch_style")
    def on_switch_style(self) -> None:
        """Set the next available style."""
        self._current_idx += 1
        if self._current_idx == len(THEMES.keys()):
            self._current_idx = 0
        style = list(THEMES.keys())[self._current_idx]
        self.on_change_theme(style)



"""Style handling for QuteStyleWindow."""
import logging
from typing import Dict, Optional

from PyQt5.QtCore import QRect, QSettings, QSize
from PyQt5.QtGui import QColor, QPainter, QPixmap

log = logging.getLogger(
    f"qute_style.{__name__}"
)  # pylint: disable=invalid-name


# Use this variable when referencing a default style so that adapting to a new
# default style will only require changes in the lib.
DEFAULT_STYLE = "Darcula"

CURRENT_STYLE: Optional[str] = None


def _create_theme_drawing(
    icon_size: QSize, color_names: Dict[str, str]
) -> QPixmap:
    """Create theme drawing."""
    pixmap = QPixmap(icon_size)
    painter = QPainter(pixmap)
    # draw background
    painter.fillRect(
        QRect(0, 0, pixmap.width(), pixmap.height()),
        QColor(color_names["bg_one"]),
    )
    # draw menu
    menu = QRect(0, 0, 20, pixmap.height())
    painter.fillRect(menu, QColor(color_names["dark_one"]))
    # draw menu icons
    for i in range(8):
        y_pos = i * 20 + 5
        if y_pos + 8 < menu.height():
            painter.fillRect(
                QRect(5, i * 20 + 5, 8, 8),
                QColor(color_names["active"]),
            )
    # draw toolbar
    toolbar = QRect(menu.width() + 5, 0, pixmap.width(), 15)
    painter.fillRect(toolbar, QColor(color_names["bg_two"]))

    # draw footer
    footer = QRect(menu.width() + 5, pixmap.height() - 10, pixmap.width(), 10)
    painter.fillRect(footer, QColor(color_names["bg_two"]))

    # draw widget
    widget = QRect(
        menu.width() + 5,
        toolbar.height() + 5,
        pixmap.width() - (menu.width() + 5),
        pixmap.height() - (toolbar.height() + footer.height() + 10),
    )
    painter.fillRect(widget, QColor(color_names["bg_two"]))

    # draw widget data
    for i in range(6):
        y_pos = i * 20 + 10
        if y_pos + 5 < widget.height():
            if i % 2:
                width = widget.width() - 40
                color = QColor(color_names["foreground"])
            else:
                width = widget.width() - 100
                color = QColor(color_names["context_color"])

            painter.fillRect(
                QRect(widget.x() + 10, y_pos + widget.y(), width, 5),
                color,
            )
    painter.end()
    return pixmap


def get_current_style() -> str:
    """Return the currently set style."""
    global CURRENT_STYLE  # pylint: disable=global-statement
    if CURRENT_STYLE is None:
        CURRENT_STYLE = QSettings().value("style", DEFAULT_STYLE)
        log.debug("Loaded current style from registry: %s", CURRENT_STYLE)
        if CURRENT_STYLE not in THEMES:
            log.warning("Invalid style stored in registry: %s", CURRENT_STYLE)
            # If an invalid style is set, revert to DEFAULT_STYLE
            CURRENT_STYLE = DEFAULT_STYLE
    return CURRENT_STYLE


def set_current_style(style: str) -> None:
    """
    Set the current style.

    One should only use this method to change the style. This will correctly
    set CURRENT_STYLE to be used as a lazy variable.
    """
    log.debug("Setting current style to %s", style)
    global CURRENT_STYLE  # pylint: disable=global-statement
    CURRENT_STYLE = style
    QSettings().setValue("style", style)


THEMES: Dict[str, Dict[str, str]] = {
    "Snow White": {
        "dark_one": "#b5c3dd",
        "dark_two": "#bfcde6",
        "bg_one": "#c9d7ef",
        "bg_two": "#d3e0f7",
        "bg_elements": "#e2e9f7",
        "bg_three": "#eff1f7",
        "bg_disabled": "#e5eaf4",
        "fg_disabled": "#adaeaf",
        "foreground": "#24263f",
        "active": "#121320",
        "context_pressed": "#90a5c7",
        "context_color": "#9bb1d0",
        "context_hover": "#b2c3d6",
        "white": "#f5f6f9",
        "pink": "#ff007f",
        "green": "#15c72a",
        "light_green": "#46ff5c",
        "dark_green": "#0b6315",
        "red": "#ff5555",
        "light_red": "#ffd4d4",
        "dark_red": "#7f2a2a",
        "yellow": "#fda600",
        "light_yellow": "#ffd27c",
        "dark_yellow": "#7e5300",
        "grey": "#d3d3d3",
    },
    "Princess Pink": {
        "active": "#fffefe",
        "dark_one": "#282a36",
        "dark_two": "#363948",
        "bg_elements": "#595D75",
        "bg_one": "#44475a",
        "bg_two": "#4f5268",
        "bg_three": "#63677d",
        "bg_disabled": "#585c6e",
        "fg_disabled": "#83849b",
        "context_color": "#ff79c6",
        "context_hover": "#ffacdc",
        "context_pressed": "#e86eb4",
        "foreground": "#f6e2f6",
        "white": "#f5f6f9",
        "pink": "#ff79c6",
        "green": "#00ff7f",
        "light_green": "#7fffbf",
        "dark_green": "#007f3f",
        "red": "#ff5555",
        "light_red": "#ffd4d4",
        "dark_red": "#7f2a2a",
        "yellow": "#f1fa8c",
        "light_yellow": "#ffffff",
        "dark_yellow": "#787d46",
        "grey": "#d3d3d3",
    },
    "Darcula": {
        "active": "#dfe4ed",
        "dark_one": "#1b1e23",
        "dark_two": "#242830",
        "bg_elements": "#3c4454",
        "bg_one": "#2c313c",
        "bg_two": "#343b48",
        "bg_three": "#444C5B",
        "bg_disabled": "#3e4150",
        "fg_disabled": "#606478",
        "context_color": "#568af2",
        "context_hover": "#81A8F6",
        "context_pressed": "#4e7ddc",
        "foreground": "#b0b7c7",
        "white": "#f5f6f9",
        "pink": "#ff007f",
        "green": "#00ff7f",
        "light_green": "#7fffbf",
        "dark_green": "#007f3f",
        "red": "#ff5555",
        "light_red": "#ffd4d4",
        "dark_red": "#7f2a2a",
        "yellow": "#f1fa8c",
        "light_yellow": "#ffffff",
        "dark_yellow": "#787d46",
        "grey": "#d3d3d3",
    },
    "Highbridge Gray": {
        "active": "#0e0e0e",
        "bg_disabled": "#d9d9d9",
        "bg_one": "#ffffff",
        "bg_three": "#e3e3e3",
        "bg_two": "#f7f7f7",
        "context_color": "#aaa7bf",
        "context_hover": "#bdbcc8",
        "context_pressed": "#9e9cb7",
        "bg_elements": "#ededed",
        "dark_two": "#dedede",
        "dark_green": "#007f3f",
        "dark_one": "#c6c6c6",
        "dark_red": "#7f2a2a",
        "dark_yellow": "#787d46",
        "fg_disabled": "#919191",
        "green": "#00ff7f",
        "grey": "#d3d3d3",
        "light_green": "#7fffbf",
        "light_red": "#ffd4d4",
        "light_yellow": "#ffffff",
        "pink": "#ff007f",
        "red": "#ff5555",
        "foreground": "#393939",
        "white": "#f5f6f9",
        "yellow": "#fda600",
    },
    "Ruby Red": {
        "dark_one": "#431e1e",
        "dark_two": "#4f2424",
        "bg_one": "#5f2d2d",
        "bg_two": "#6a3030",
        "bg_elements": "#773535",
        "bg_three": "#823d3d",
        "bg_disabled": "#783a3a",
        "fg_disabled": "#a95454",
        "foreground": "#cb8d8d",
        "active": "#f4dfdf",
        "context_pressed": "#cc4848",
        "context_color": "#e44e4e",
        "context_hover": "#ea7171",
        "dark_green": "#007f3f",
        "dark_red": "#7f2a2a",
        "dark_yellow": "#787d46",
        "green": "#00ff7f",
        "grey": "#d3d3d3",
        "light_green": "#7fffbf",
        "light_red": "#ffd4d4",
        "light_yellow": "#ffffff",
        "pink": "#ff007f",
        "red": "#ff5555",
        "white": "#f5f6f9",
        "yellow": "#f1fa8c",
    },
}


def get_style() -> str:
    """Return the current style sheet that is stored in QSettings."""
    # Use the Darcula style if not style is stored yet as default.
    log.debug("Stored style: %s", get_current_style())
    return MAIN_STYLE.format(**THEMES[get_current_style()])


def get_color(name: str) -> str:
    """Return the color code for the given name."""
    return THEMES[get_current_style()][name]


MAIN_STYLE = """
/* QComboBox */
QComboBox{{
    color: {foreground};
    background-color: {bg_elements};
    border-radius: 5px;
    border: 1px solid {bg_elements};
    padding: 5px;
    padding-left: 10px;
}}
QComboBox:hover{{
    border: 1px solid {context_hover};
}}
QComboBox::drop-down {{
    width: 25px;
    border-left-width: 3px;
    border-left-color: {bg_two};
    border-left-style: solid;
    border-top-right-radius: 3px;
    border-bottom-right-radius: 3px;
}}
QComboBox QAbstractItemView {{
    color: {foreground};
    background-color: {bg_elements};
    padding: 10px;
    selection-background-color: {context_color};
}}
QComboBox:disabled {{
    color: {fg_disabled};
    background-color: {bg_disabled};
    border: {bg_disabled};
}}
QComboBox[cssClass="transparent"] {{
    background-color: transparent;
}}
QComboBox::drop-down[cssClass="transparent"] {{
    border-left-color: transparent;
}}
QComboBox:disabled[cssClass="transparent"] {{
    color: {fg_disabled};
    background-color: transparent;
    border: {bg_disabled};
}}

/* QHeaderView */
QHeaderView{{
    background-color: {bg_two};
}}
QHeaderView::section{{
    color: {foreground};
    background-color: {bg_two};
    max-width: 30px;
    border: 0px;
    padding: 3px;
}}
QHeaderView::section:horizontal{{
    border-right: 1px solid {dark_two};
    border-right: 1px solid {dark_two};
    border-bottom: 1px solid {dark_two};
}}
QHeaderView::section:horizontal:only-one{{
    border-right: 0px;
}}
QHeaderView::section:horizontal:last{{
    border-right: 0px;
}}
QHeaderView::section:vertical{{
    border-bottom: 1px solid {dark_two};
    border-right: 1px solid {dark_two};
}}
QHeaderView::section:vertical:last{{
    border-bottom: 0px;
}}
QHeaderView::section:vertical:only-one{{
    border-bottom: 0px;
}}
QHeaderView::section:disabled{{
    color: {fg_disabled};
}}
QTableCornerButton::section {{
    background-color: {bg_two};
}}

/* QLineEdit */
QLineEdit,
QPlainTextEdit {{
    background-color: {bg_elements};
    border-radius: 8px;
    border: 1px solid transparent;
    padding-left: 10px;
    padding-right: 10px;
    selection-color: {active};
    selection-background-color: {context_color};
    color: {foreground};
    height: 30px;
}}
QLineEdit:read-only, QPlainTextEdit:read-only{{
    background-color: {bg_disabled};
    color: {fg_disabled};
}}
QLineEdit:read-only:focus,
QPlainTextEdit:read-only:focus {{
    border: 1px solid {bg_disabled};
    background-color: {bg_disabled};
}}

QLineEdit#column_line_edit {{
    background-color: {bg_elements};
    border-radius: 8px;
    border: 1px solid transparent;
    padding-left: 10px;
    padding-right: 10px;
    selection-color: {active};
    selection-background-color: {context_color};
    color: {foreground};
}}
QLineEdit:focus,
QPlainTextEdit:focus {{
    border: 1px solid {context_color};
    background-color: {bg_one};
}}
QLineEdit#column_line_edit:focus {{
    border: 1px solid {context_color};
    background-color: {bg_two};
}}

/* QMenu */
QMenu{{
    background-color: {bg_one};
    color: {foreground};
}}
QMenu::item:disabled {{
    color: {fg_disabled};
}}
QMenu::item:enabled:selected {{
    color: {active};
    background-color: {context_color};
}}
QMenu QCheckBox{{
    color: {foreground};
    border-radius: 17px;
    border: 10px solid transparent;
}}
QMenu QCheckBox:hover {{
    background-color: {context_hover};
    color: {active};
}}
QMenu QCheckBox:pressed {{
    background-color: {context_pressed};
    color: {active};
}}
QMenu::separator {{
    background: {bg_three};
    height: 3px;
    margin-left: 5px;
    margin-right: 5px;
}}
/* QScrollArea */
QScrollArea {{
    border: none;
}}
QWidget#scroll_widget {{
    background: {dark_one};
}}

QWidget#style_selection_widget
        {{ background-color:{bg_one}; }}

/* QProgressBar */
QProgressBar {{
    background-color: {bg_elements};
    color: {foreground};
    border-style: none;
    border-radius: 10px;
    text-align: center;
}}
QProgressBar::chunk {{
    background-color: {context_color};
    border-radius: 10px;
}}

/* QScrollBar */
QScrollBar:horizontal {{
    border: none;
    background: {bg_three};
    height: 8px;
    margin: 0px 21px 0 21px;
    border-radius: 0px;
}}
QScrollBar:horizontal:disabled {{
    background: {bg_disabled};
}}
QScrollBar::handle:horizontal {{
    background: {context_color};
    min-width: 25px;
    border-radius: 4px
}}
QScrollBar::handle:horizontal:disabled {{
    background: {fg_disabled};
}}
QScrollBar::handle:horizontal:hover {{
    background: {context_hover};
    min-width: 25px;
    border-radius: 4px
}}
QScrollBar::handle:horizontal:pressed {{
    background: {context_pressed};
    min-width: 25px;
    border-radius: 4px
}}
QScrollBar::add-line:horizontal {{
    border: none;
    background: {dark_two};
    width: 20px;
    border-top-right-radius: 4px;
    border-bottom-right-radius: 4px;
    subcontrol-position: right;
    subcontrol-origin: margin;
}}
QScrollBar::sub-line:horizontal {{
    border: none;
    background: {dark_two};
    width: 20px;
    border-top-left-radius: 4px;
    border-bottom-left-radius: 4px;
    subcontrol-position: left;
    subcontrol-origin: margin;
}}
QScrollBar::up-arrow:horizontal, QScrollBar::down-arrow:horizontal
{{
     background: none;
}}
QScrollBar::add-page:horizontal, QScrollBar::sub-page:horizontal
{{
     background: none;
}}
QScrollBar:vertical {{
    border: none;
    background: {bg_three};
    width: 8px;
    margin: 21px 0 21px 0;
    border-radius: 0px;
}}
QScrollBar:vertical:disabled {{
    background: {bg_disabled};
}}
QScrollBar::handle:vertical {{
    background: {context_color};
    min-height: 25px;
    border-radius: 4px
}}
QScrollBar::handle:vertical:disabled {{
    background: {fg_disabled};
}}
QScrollBar::handle:vertical:hover {{
    background: {context_hover};
    min-height: 25px;
    border-radius: 4px
}}
QScrollBar::handle:vertical:pressed {{
    background: {context_pressed};
    min-height: 25px;
    border-radius: 4px
}}
QScrollBar::add-line:vertical {{
    border: none;
    background: {dark_two};
    height: 20px;
    border-bottom-left-radius: 4px;
    border-bottom-right-radius: 4px;
    subcontrol-position: bottom;
    subcontrol-origin: margin;
}}
QScrollBar::sub-line:vertical {{
    border: none;
    background: {dark_two};
    height: 20px;
    border-top-left-radius: 4px;
    border-top-right-radius: 4px;
    subcontrol-position: top;
    subcontrol-origin: margin;
}}
QScrollBar::up-arrow:vertical, QScrollBar::down-arrow:vertical {{
     background: none;
}}
QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical {{
     background: none;
}}

/* QSlider */
QSlider {{
    margin: 0px;
}}
QSlider::groove:horizontal {{
    border-radius: 10px;
    height: 20px;
    margin: 0px;
    background-color: {bg_elements};
}}
QSlider::groove:horizontal:hover {{
    background-color: {bg_elements};
}}
QSlider::handle:horizontal {{
    border: none;
    height: 16px;
    width: 16px;
    margin: 2px;
    border-radius: 8px;
    background-color: {context_color};
}}
QSlider::handle:horizontal:hover {{
    background-color: {context_hover};
}}
QSlider::handle:horizontal:pressed {{
    background-color: {context_pressed};
}}
QSlider::groove:vertical {{
    border-radius: 10px;
    width: 20px;
    margin: 0px;
    background-color: {bg_elements};
}}
QSlider::groove:vertical:hover {{
    background-color: {bg_elements};
}}
QSlider::handle:vertical {{
    border: none;
    height: 16px;
    width: 16px;
    margin: 2px;
    border-radius: 8px;
    background-color: {context_color};
}}
QSlider::handle:vertical:hover {{
    background-color: {context_hover};
}}
QSlider::handle:vertical:pressed {{
    background-color: {context_pressed};
}}

/* QSplitter */

/* This activates the hover which isn't active by default */
QSplitterHandle:hover {{
}}

/* QSplitter can only be correctly addressed by it's orientation property.
Also, one must take care that the splitter in vertical direction is actually
turned by 90 degrees, so you'll need to use i.e. width as height etc. */

/* Horizontal QSplitter */
QSplitter[orientation='1']::handle {{
    height: 2px;
    background-color: {bg_elements};
}}
QSplitter[orientation='1']::handle:hover {{
    background-color: {context_color};
}}

/* Vertical QSplitter */
QSplitter[orientation='2']::handle {{
    height: 2px;
    background-color: {bg_elements};
}}
QSplitter[orientation='2']::handle:hover {{
    background-color: {context_color};
}}

/* QTabWidget */
/*
QTabWidget lacks of proper qss background-color support.
See:
https://bugreports.qt.io/browse/QTBUG-33344
https://bugreports.qt.io/browse/QTBUG-68642
https://codereview.qt-project.org/c/qt/qtbase/+/230769/
Because of not inheriting the values properly each
widget of QTabWidget needs to be set manually.
*/
QTabWidget > QStackedWidget {{
    background-color: {bg_two};
    border-top-right-radius: 8px;
    border-bottom-left-radius: 8px;
    border-bottom-right-radius: 8px;
}}
QTabWidget > QStackedWidget > QWidget {{
    background-color: {bg_two};
    border-top-right-radius: 8px;
    border-bottom-left-radius: 8px;
    border-bottom-right-radius: 8px;
}}
 QTabWidget::pane {{
    background-color: {bg_two};
    border-top-right-radius: 8px;
    border-bottom-left-radius: 8px;
    border-bottom-right-radius: 8px;
}}

/* QTabBar */
QTabBar::tab {{
    background-color: {bg_three};
    color: {foreground};
    border-top-left-radius: 8px;
    border-top-right-radius: 8px;
    padding: 12px;
}}
QTabBar::tab:hover {{
    background-color: {context_color};
    color: {active};
}}
QTabBar::tab:selected {{
    background-color: {bg_two};
    color: {foreground};
}}

/* QTextEdit */
QTextEdit {{
    color: {foreground};
    background-color: {bg_elements};
    padding: 10px;
    border-radius: 5px;
}}
QTextEdit:disabled {{
    color: {fg_disabled};
    background-color: {bg_disabled};
}}

/* QToolTip */
QToolTip {{
    background-color: {dark_one};
    color:{foreground};
    padding-left: 10px;
    padding-right: 10px;
    border: 0px solid transparent;
    border-left: 3px solid {context_color};
}}

/* QTreeView, QListView, QTableView, QTableWidget, QTreeWidget */
QTreeView,
QListView,
QTableView,
QTableWidget,
QTreeWidget {{
    color: {foreground};
    background-color: {bg_two};
    alternate-background-color: {bg_one};
    padding: 10px;
    border-radius: 5px;
}}
#frozen_column_table_view {{
    /* This QTableView must fix exactly at it's position over the real table */
    padding: 0px;

}}
#frozen_column_table_view QHeaderView::section:horizontal:last{{
    /* The last section of the frozen table's header isn't the real last one */
    border-right: 1px solid {dark_two};
}}
QTreeView:disabled,
QListView:disabled,
QTableView:disabled,
QTableWidget:disabled,
QTreeWidget:disabled {{
    color: {fg_disabled};
}}

QTableView::item QComboBox{{
    border-radius: 0px;
}}


QFrame #_left_column_frame {{
    background-color: {bg_two};
}}

QFrame#app_background {{
    background-color: {bg_one};
    border-color: {bg_two};
    border-style: solid;
    /* Those are the default values that are applied when the app is not
    maximized */
    border-radius: 10;
    border: 2px;
}}
QFrame {{
    color: {foreground};
    font: 9pt 'Segoe UI';
}}
/* Used for Frames, and other space covering widgets like title bar,
credit bar and widgets in the right column (mostly QFrame) */
#bg_two_frame {{
    background-color: {bg_two};
    border-radius: 8px;
}}
QFrame#title_bg_frame {{
    background-color: {bg_one};
    border-radius: 8px;
}}

QFrame#div {{
    background: {dark_two};
    border-radius: 0;
}}

.QLabel {{
    font: 9pt "Segoe UI";
    color: {foreground};
    padding-left: 10px;
    padding-right: 10px;
}}
/* Label that has no padding on the left side */
QLabel#left_label {{
    font: 9pt "Segoe UI";
    color: {foreground};
    padding-left: 0px;
    padding-right: 10px;
}}
QLabel#heading_label {{
    font: 14pt "Segoe UI";
}}
QLabel#db_label {{
    color: red;
    font-weight: bold;
}}
QLabel:disabled {{
    color: {fg_disabled};
}}
QLabel#heading_label {{
    font: 14pt "Segoe UI";
}}
QLabel#heading1_label {{
    font: 12pt "Segoe UI";
    background: {dark_one};
    border-radius: 8px;
}}
QLabel#heading2_label {{
    font: 10pt "Segoe UI";
    background: {bg_two};
    border-radius: 8px;
}}
/* Completer */
#completer_popup{{
    border: 1px;
    border-color: {context_pressed};
    border-style: solid;
}}
QListView#completer_popup{{
    padding: 0px 10px 0px 10px;
}}

/* Background around the LeftMenu */
QFrame#menu_background {{
    background-color: {dark_one};
    border-radius: 8px;
}}

QLabel#column_title_label {{
    font-size: 10pt;
    color: {foreground};
    padding-bottom: 2px;
}}

/* CornerGrip and EdgeGrip are transparent. */
#grip {{
    background: transparent;
}}

/* Style for tooltips on buttons. */
QLabel#label_tooltip {{
    background-color: {dark_one};
    color: {foreground};
    padding-left: 10px;
    padding-right: 10px;
    border-radius: 17px;
    border: 0px solid transparent;
    border-left: 3px solid {context_color};
    font: 800 9pt "Segoe UI";
}}

QLabel#title_label {{
    font: 10pt "Segoe UI";
    color: {foreground};
}}
QLabel#about_toolbox {{
    font: 9pt "Segoe UI";
    color: {foreground};
}}
QMessageBox{{
    background-color: {dark_two};
    border-left: 3px solid {context_color};
    font: 9pt "Segoe UI";
    color: {foreground};
}}
QPushButton{{
    background-color: {bg_elements};
    color: {active};
    padding-left: 10px;
    padding-right: 10px;
    border-radius: 17px;
    border: 10px solid transparent;
    font: 600 9pt "Segoe UI";
}}
QPushButton:hover {{
    background-color: {context_color};
}}
QPushButton:pressed {{
    background-color: {context_pressed};
}}
QPushButton:checked {{
    background-color: {context_pressed};
}}
QPushButton#info_widget_btn{{
    background-color: {dark_one};
    color: {foreground};
    padding-left: 10px;
    padding-right: 10px;
    border-radius: 17px;
    border: 10px solid transparent;
    border-left: 3px solid {context_color};
    font: 800 9pt "Segoe UI";
}}

QPushButton[cssClass="red"] {{ background-color: {red}; color: black;}}
QPushButton[cssClass="red"]:hover {{ background-color: {light_red};
color: black;}}
QPushButton[cssClass="red"]:pressed {{ background-color: {dark_red};
color: black;}}

QPushButton[cssClass="green"] {{ background-color: {green};
color: black;}}
QPushButton[cssClass="green"]:hover {{ background-color: {light_green};
color: black;}}
QPushButton[cssClass="green"]:pressed {{ background-color: {dark_green};
color: black;}}

QPushButton[cssClass="yellow"] {{ background-color: {yellow};
color: black;}}
QPushButton[cssClass="yellow"]:hover {{ background-color: {light_yellow};
color: black;}}
QPushButton[cssClass="yellow"]:pressed {{ background-color: {dark_yellow};
color: black;}}

QPushButton#info_widget_btn:hover {{
    background-color: {dark_two};
}}
QPushButton#info_widget_btn:pressed {{
    background-color: {dark_one};
}}
QPushButton#info_widget_btn:disabled{{
    color: {fg_disabled};
    background-color: {bg_disabled};
}}
    """



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'qute_style\ui\whats_new_window.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_whats_new_window(object):
    def setupUi(self, whats_new_window):
        whats_new_window.setObjectName("whats_new_window")
        whats_new_window.resize(389, 341)
        self.centralwidget = QtWidgets.QWidget(whats_new_window)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.scrollArea = QtWidgets.QScrollArea(self.centralwidget)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 369, 249))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(
            self.scrollAreaWidgetContents
        )
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.text_label = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.text_label.setText("")
        self.text_label.setWordWrap(True)
        self.text_label.setObjectName("text_label")
        self.horizontalLayout_3.addWidget(self.text_label)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout.addWidget(self.scrollArea)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(
            40,
            20,
            QtWidgets.QSizePolicy.Expanding,
            QtWidgets.QSizePolicy.Minimum,
        )
        self.horizontalLayout.addItem(spacerItem)
        self.previous_button = QtWidgets.QPushButton(self.centralwidget)
        self.previous_button.setObjectName("previous_button")
        self.horizontalLayout.addWidget(self.previous_button)
        self.next_button = QtWidgets.QPushButton(self.centralwidget)
        self.next_button.setObjectName("next_button")
        self.horizontalLayout.addWidget(self.next_button)
        self.close_button = QtWidgets.QPushButton(self.centralwidget)
        self.close_button.setObjectName("close_button")
        self.horizontalLayout.addWidget(self.close_button)
        self.verticalLayout.addLayout(self.horizontalLayout)
        whats_new_window.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(whats_new_window)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 389, 21))
        self.menubar.setObjectName("menubar")
        whats_new_window.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(whats_new_window)
        self.statusbar.setObjectName("statusbar")
        whats_new_window.setStatusBar(self.statusbar)

        self.retranslateUi(whats_new_window)
        QtCore.QMetaObject.connectSlotsByName(whats_new_window)

    def retranslateUi(self, whats_new_window):
        _translate = QtCore.QCoreApplication.translate
        whats_new_window.setWindowTitle(
            _translate("whats_new_window", "StyledMainWindow")
        )
        self.previous_button.setToolTip(
            _translate("whats_new_window", "Den letzten Eintrag anzeigen.")
        )
        self.previous_button.setText(_translate("whats_new_window", "Zurück"))
        self.next_button.setToolTip(
            _translate("whats_new_window", "Den nächsten Eintrag anzeigen.")
        )
        self.next_button.setText(_translate("whats_new_window", "Weiter"))
        self.close_button.setToolTip(
            _translate(
                "whats_new_window",
                'Das Fenster schließen. Sie können das Fenster erneut anzeigen, indem Sie im Menü "Hilfe" den Eintrag "Neuerung" anwählen.',
            )
        )
        self.close_button.setText(_translate("whats_new_window", "Schließen"))



"""Top bar with system buttons and extra menu."""
import logging
from typing import List, Type, cast

from PyQt5.QtCore import QEvent, QObject, QPoint, Qt, pyqtSignal, pyqtSlot
from PyQt5.QtGui import QMouseEvent
from PyQt5.QtWidgets import QFrame, QHBoxLayout, QLabel, QSizePolicy, QWidget

from qute_style.widgets.base_widgets import BaseWidget
from qute_style.widgets.icon import Icon
from qute_style.widgets.title_button import TitleButton

log = logging.getLogger(
    f"qute_style.{__name__}"
)  # pylint: disable=invalid-name


class TitleBar(QFrame):
    """Top bar with system buttons and extra menu."""

    right_button_clicked = pyqtSignal(type, name="right_button_clicked")
    close_app = pyqtSignal(name="close_app")
    minimize = pyqtSignal(name="minimize")
    maximize = pyqtSignal(name="maximize")
    move_window = pyqtSignal(QPoint, name="move")

    def __init__(  # pylint: disable=too-many-arguments
        self,
        parent: QWidget,
        app_parent: QWidget,
        right_widget_classes: List[Type[BaseWidget]],
        name: str,
        logo: str,
        debug_text: str = "",
    ) -> None:
        """Create a new TitleBar."""
        super().__init__(parent)
        self.setObjectName("bg_two_frame")

        # Fix the height of the TitleBar to 40 pixels.
        self.setFixedHeight(40)

        bg_layout = QHBoxLayout(self)
        bg_layout.setContentsMargins(10, 0, 5, 0)
        bg_layout.setSpacing(0)

        # Icon with ToolBox logo
        self._icon = Icon(int(self.height() * 0.5), None)
        self._icon.set_icon(logo)
        self._icon.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        bg_layout.addWidget(self._icon)

        # Label with the application title text.
        self._title_label = QLabel()
        self._title_label.setObjectName("title_label")
        self._title_label.setAlignment(Qt.AlignVCenter)
        self._title_label.setText(name)
        bg_layout.addWidget(self._title_label)

        # eventFilter is used to enable
        # moving + min/maximizing of the app itself.
        self._icon.installEventFilter(self)
        self._title_label.installEventFilter(self)

        # set debug label if any
        debug_label = QLabel(debug_text)
        debug_label.setObjectName("db_label")
        debug_label.setAlignment(Qt.AlignVCenter)
        debug_label.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        bg_layout.addWidget(debug_label)

        # Button for the right column
        for widget_class in right_widget_classes:
            right_column_button = TitleButton(
                app_parent,
                tooltip_text=widget_class.NAME,
                icon_path=widget_class.ICON,
                widget_class=widget_class,
            )
            right_column_button.clicked.connect(self.on_right_column_button)
            bg_layout.addWidget(right_column_button)

        # Minimize Button
        minimize_button = TitleButton(
            app_parent,
            tooltip_text=self.tr("Minimieren"),
            icon_path=":/svg_icons/minimize.svg",
        )
        minimize_button.released.connect(self.minimize)
        bg_layout.addWidget(minimize_button)

        # Maximize Button
        self.maximize_button = TitleButton(
            app_parent,
            tooltip_text=self.tr("Maximieren"),
            icon_path=":/svg_icons/fullscreen.svg",
        )
        self.maximize_button.released.connect(self.maximize)
        bg_layout.addWidget(self.maximize_button)

        # Close Button
        close_button = TitleButton(
            app_parent,
            tooltip_text=self.tr("Schließen"),
            icon_path=":/svg_icons/close.svg",
        )
        close_button.released.connect(self.close_app)
        bg_layout.addWidget(close_button)

        self._double_click_in_progress = False

    @property
    def title_bar_text(self) -> str:
        """Get the title bar text."""
        return self._title_label.text()

    @title_bar_text.setter
    def title_bar_text(self, text: str) -> None:
        """Set the title bar text."""
        self._title_label.setText(text)

    def eventFilter(  # pylint: disable=invalid-name
        self, obj: QObject, event: QEvent
    ) -> bool:
        """
        Handle mouse events.

        MouseButtonDblClick is a combination of MousePress, MouseRelease
        and MousePress. The final Mouse Release is not part of this event.
        Thus it can happen that by executing a MouseButtonDblClick also a Mouse
        MoveEvent is executed. The introduced _double_click_in_progress
        variable blocks this situation which could lead to e.g. a maximize
        operation followed directly by a minimize operation.
        """
        if obj is not self._title_label and not self._icon:
            return False
        if event.type() == QEvent.MouseButtonRelease:
            self._double_click_in_progress = False
            return True
        if event.type() == QEvent.MouseButtonDblClick:
            self._double_click_in_progress = True
            self.maximize.emit()
            return True
        if event.type() == QEvent.MouseMove:
            if not self._double_click_in_progress:
                self.move_window.emit(cast(QMouseEvent, event).globalPos())
            return True
        return False

    def set_maximized(self, maximized: bool) -> None:
        """Set the _background icon depending if the app is maximized."""
        name = "fullscreen_exit" if maximized else "fullscreen"
        self.maximize_button.set_icon(f":/svg_icons/{name}.svg")
        if maximized:
            self.maximize_button.tooltip_text = self.tr("Verkleinern")
        else:
            self.maximize_button.tooltip_text = self.tr("Maximieren")

    @pyqtSlot(name="on_right_column_button")
    def on_right_column_button(self) -> None:
        """Handle a click on one of the buttons for left column widgets."""
        widget_class = cast(TitleButton, self.sender()).widget_class
        log.debug("Emitting right_button_clicked for class %s", widget_class)
        self.right_button_clicked.emit(widget_class)

    def set_button_active(
        self,
        widget_class: Type[BaseWidget],
        active: bool,
    ) -> None:
        """Set the button for the given widget active/inactive."""
        button = self._button(widget_class)
        button.set_active(active)

    def _button(self, widget_class: Type[BaseWidget]) -> TitleButton:
        """Return the button for the given widget class."""
        for btn in self.findChildren(TitleButton):
            if btn.widget_class == widget_class:
                return btn
        raise ValueError(  # pragma: no cover
            f"Could not find button for widget: {widget_class}"
        )



"""Sample widgets."""
from __future__ import annotations

from pathlib import Path
from typing import Dict, List, cast

from PyQt5.QtCore import (
    QEvent,
    QFileInfo,
    QModelIndex,
    QObject,
    QSize,
    QStringListModel,
    Qt,
    pyqtSlot,
)
from PyQt5.QtGui import QDragEnterEvent, QDropEvent, QIcon
from PyQt5.QtWidgets import (
    QCheckBox,
    QDialogButtonBox,
    QFileIconProvider,
    QHBoxLayout,
    QLabel,
    QListView,
    QMenu,
    QSizePolicy,
    QSpacerItem,
    QTreeWidgetItem,
    QVBoxLayout,
    QWidget,
    QWidgetAction,
)

from qute_style.gen.ui_test_window import Ui_test_widget
from qute_style.helper import create_tooltip, create_waiting_spinner
from qute_style.widgets.base_widgets import MainWidget
from qute_style.widgets.custom_icon_engine import CustomIconEngine
from qute_style.widgets.drop_label import DropLabel


class TestWidget(MainWidget):
    """Test Widget."""

    ICON = ":/svg_icons/heart_broken.svg"
    NAME = "Test-Widget"

    def __init__(self, parent: QWidget | None = None) -> None:
        """Create a new TestWidget."""
        super().__init__(parent)
        self._ui = Ui_test_widget()
        self._ui.setupUi(self)
        heart_path = ":/svg_icons/heart_broken.svg"
        icon = QIcon(CustomIconEngine(heart_path, "foreground"))
        self._ui.custom_icon_engine_checkbox.setIcon(icon)
        self._ui.custom_icon_engine_checkbox.setIconSize(QSize(16, 16))
        self._ui.icon_checkbox.setIcon(QIcon(heart_path))
        self._ui.icon_checkbox.setIconSize(QSize(16, 16))
        menu = QMenu(self._ui.pushButton_2)
        all_checkbox = QCheckBox(self.tr("Select everything"))
        widget_action = QWidgetAction(menu)
        widget_action.setDefaultWidget(all_checkbox)
        menu.addAction(widget_action)
        menu.addSeparator()
        buttons = QDialogButtonBox(
            QDialogButtonBox.Ok | QDialogButtonBox.Cancel,
            Qt.Horizontal,
            menu,
        )
        buttons.setCenterButtons(True)
        widget_action = QWidgetAction(menu)
        widget_action.setDefaultWidget(buttons)
        menu.addAction(widget_action)
        self._ui.disable_widgets.clicked.connect(self.on_widgets_disabled)
        self._ui.transparent_combobox.setProperty("cssClass", "transparent")
        self._ui.pushButton_2.setMenu(menu)
        self._ui.pushButton_2.setIcon(icon)
        self._ui.pushButton_4.set_icon(":/svg_icons/accept.svg")
        self._ui.splitter_button.setText("Change orientation")
        self._ui.splitter_button.clicked.connect(self.on_change_orientation)
        self._ui.horizontalSlider.valueChanged.connect(
            self._ui.progressBar.setValue
        )

        text = self.tr("Drop some files.")
        self._drop_label = DropLabel(text, self._ui.drop_widget)
        self._ui.drop_widget.installEventFilter(self)
        self._ui.clear_drop_button.clicked.connect(self.on_clear_drop)

        title = self.tr("Hello there")
        text = self.tr("This might be quite useful.")
        for child in self.children():
            if hasattr(cast(QWidget, child), "toolTip"):
                cast(QWidget, child).setToolTip(create_tooltip(title, text))

    def eventFilter(  # pylint: disable=invalid-name
        self, obj: QObject, event: QEvent
    ) -> bool:
        """Filter and handle QDragEnterEvents and QDropEvents."""
        if event.type() == QEvent.Drop and obj is self._ui.drop_widget:
            self._handle_file_drop(cast(QDropEvent, event))
            return True
        if (
            event.type() == QEvent.DragEnter
            and cast(QDragEnterEvent, event).mimeData().hasUrls()
        ):
            return self._handle_drag_event(cast(QDragEnterEvent, event), obj)
        return False

    def _handle_drag_event(self, event: QDragEnterEvent, obj: QObject) -> bool:
        """Handle a QDragEnterEvent."""
        if obj is self._ui.drop_widget:
            event.acceptProposedAction()
            return True
        return False  # pragma: no cover

    def _handle_file_drop(self, event: QDropEvent) -> None:
        """Handle a QDropEvent on the list of files."""
        files = event.mimeData().urls()
        for file in files:
            path = Path(file.toLocalFile())
            if path.is_dir():
                return
            self._add_file(path)

    def _add_file(self, path: Path) -> None:
        """Add a file to the list of files."""
        self._drop_label.hide()

        # Create a QTreeWidgetItem that displays the filename only.
        item = QTreeWidgetItem(self._ui.drop_widget, [path.name])
        item.setData(0, Qt.UserRole, path)

        # Get a QIcon for the file from OS and set it.
        item.setIcon(0, QFileIconProvider().icon(QFileInfo(str(path))))

    def on_clear_drop(self) -> None:
        """Remove all items from DropArea."""
        self._ui.drop_widget.clear()
        self._drop_label.show()

    @pyqtSlot(name="on_change_orientation")
    def on_change_orientation(self) -> None:
        """Change the orientation of the QSplitter."""
        if self._ui.splitter.orientation() == Qt.Horizontal:
            self._ui.splitter.setOrientation(Qt.Vertical)
        else:
            self._ui.splitter.setOrientation(Qt.Horizontal)

    @pyqtSlot(name="on_widgets_disabled")
    def on_widgets_disabled(self) -> None:
        """Disable all widgets."""
        if self._ui.disable_widgets.isChecked():
            for child in self.children():
                if child.objectName() != "disable_widgets":
                    cast(QWidget, child).setEnabled(False)
        else:
            for child in self.children():
                cast(QWidget, child).setEnabled(True)


class Model(QStringListModel):
    """Test model with check states."""

    def __init__(self, data: List[str], parent: QObject | None = None) -> None:
        """Create a new Model."""
        self._check_states: Dict[str, Qt.CheckState] = {}
        super().__init__(data, parent)

    def data(
        self, index: QModelIndex, role: int = Qt.DisplayRole
    ) -> Qt.CheckState:
        """Return data for the given role and index."""
        if role == Qt.CheckStateRole:
            return self._check_states[index.data()]
        return super().data(index, role)

    def setData(  # pylint: disable=invalid-name
        self, index: QModelIndex, value: Qt.CheckState, role: int = Qt.EditRole
    ) -> bool:
        """Set data for the given role and index."""
        if role == Qt.CheckStateRole:
            self._check_states[index.data()] = value
            self.dataChanged.emit(index, index, [Qt.CheckStateRole])
        return super().setData(index, value, role)

    def flags(self, _: QModelIndex) -> Qt.ItemFlags:
        """Return the flags for the given index."""
        return Qt.ItemIsEnabled | Qt.ItemIsUserCheckable | Qt.ItemIsSelectable


class ModelViewWidget(MainWidget):
    """Test Widget for Model View tests."""

    ICON = ":/svg_icons/heart_broken.svg"
    NAME = "Test-Widget"

    def __init__(self, parent: QWidget | None = None) -> None:
        """Create a new TestWidget."""
        super().__init__(parent)
        layout = QHBoxLayout(self)
        self._view = QListView()
        self._settings_widget: None | QWidget = None
        layout.addWidget(self._view)
        model = Model(["Item1", "Item2", "Item3\nwith line break"])

        for row in range(model.rowCount()):
            index = model.index(row)
            model.setData(
                index,
                Qt.Checked if row % 2 else Qt.Unchecked,  # alternate value
                Qt.CheckStateRole,
            )
        self._view.setModel(model)

    @property
    def settings_widget(self) -> QWidget:
        """Return ModelViewWidget specific settings."""
        if not self._settings_widget:
            self._settings_widget = QWidget()
            QVBoxLayout(self._settings_widget)
            self._settings_widget.layout().addWidget(
                QLabel(f"Settings {self.NAME}", self)
            )
            self._settings_widget.layout().addWidget(QCheckBox("Activate 1"))
            self._settings_widget.layout().addWidget(QCheckBox("Activate 2"))
            self._settings_widget.layout().addWidget(QCheckBox("Activate 2"))
            self._settings_widget.layout().addWidget(QLabel("...", self))
            self._settings_widget.layout().addItem(
                QSpacerItem(0, 0, QSizePolicy.Expanding, QSizePolicy.Expanding)
            )
        return self._settings_widget


class SpinnerWidget(MainWidget):
    """Test Widget for pyqtspinner."""

    ICON = ":/svg_icons/heart_broken.svg"
    NAME = "Spinner-Widget"

    def __init__(self, parent: QWidget | None = None) -> None:
        """Create a new TestWidget."""
        super().__init__(parent)
        layout = QHBoxLayout(self)
        self._view = QListView()
        self._settings_widget: None | QWidget = None
        layout.addWidget(self._view)
        self._spinner = create_waiting_spinner(self)
        self._spinner.start()



"""Close Button on the left Menu."""
from __future__ import annotations

from PyQt5.QtWidgets import QWidget

from qute_style.widgets.icon_button import BackgroundColorNames
from qute_style.widgets.title_button import TitleButton


class LeftColumnCloseButton(TitleButton):
    """Close Button on the left TitleBar (TitleButton with other colors)."""

    def __init__(
        self,
        app_parent: QWidget,
        tooltip_text: str,
        icon_path: str,
        parent: QWidget | None = None,
    ) -> None:
        """Init LeftMenuCloseButton."""
        bgs = BackgroundColorNames(
            hovering="bg_two",
            background="bg_one",
            pressed="bg_one",
            released="bg_three",
        )
        super().__init__(app_parent, tooltip_text, icon_path, parent, bgs)



"""Sample classes."""
from __future__ import annotations

from PyQt5.QtWidgets import QWidget

from qute_style.widgets.styled_combobox import CheckableComboBox


class TestComboBox(CheckableComboBox[int]):
    """Combobox that displays a list of sample items to be checked."""

    def __init__(self, parent: QWidget | None = None) -> None:
        """Create a new TestCombobox."""
        super().__init__(parent)
        self._default_text = "No selection."

        # Set the data for nine test items
        for idx in range(1, 10):
            text = f"New Item {idx}"
            self.addItem(text, idx, ":/svg_icons/delete_forever.svg")



"""Custom Tooltip for a TitleButton."""

import logging

from PyQt5.QtGui import QColor
from PyQt5.QtWidgets import QGraphicsDropShadowEffect, QLabel, QWidget

log = logging.getLogger("qute_style._tooltip")  # pylint: disable=invalid-name


class ToolTip(QLabel):
    """Custom Tooltip for a TitleButton."""

    def __init__(self, parent: QWidget, tooltip: str) -> None:
        """Create a new Tooltip."""
        super().__init__(text=tooltip, parent=parent)
        self.setObjectName("label_tooltip")
        self.setMinimumHeight(34)

        shadow = QGraphicsDropShadowEffect(self)
        shadow.setBlurRadius(30)
        shadow.setXOffset(0)
        shadow.setYOffset(0)
        shadow.setColor(QColor(0, 0, 0, 80))
        self.setGraphicsEffect(shadow)

    def show(self) -> None:
        """Show the tooltip and adjust the size before."""
        # Adjust the size. This must be done after the stylesheet was set,
        # because the font settings will change the size needed.
        self.adjustSize()
        super().show()
